//*
//******************************************************************
//* Module: Database.c
//*	This module will handle everythat needs to be performed
//*on the phone book database...



#include <windows.h>
#include <commctrl.h>
#include <dlgs.h>
#include "Globals.h"
#include "Database.h"

//
// Local prototypes
//
INT GetMaxListViewItems( void );
INT AddItem( HWND, INT, LPTSTR, LPTSTR,	LPTSTR, LPTSTR, INT );
PLVDATAITEM GetItemData( INT );
INT LoadACache( PLVDATAITEM, INT, INT );
INT LoadMainCache( INT, INT );
INT LoadTopCache( void );
INT LoadBotCache( void );
void FlushMainCache( void );
void FlushEndCaches( void );
INT InitDatabaseItem( INT, LPTSTR, LPTSTR, LPTSTR, LPTSTR, INT );
void InitDataBaseSubItem( INT, INT, LPTSTR );
void InitDatabase( void );
PLVDATAITEM GetDatabaseItem( INT );
INT SetDatabaseItem( INT, PLVDATAITEM );
BOOL IsPhoneBookLoaded( void );
void SetPhoneBookLoaded( BOOL );

//*
//******************************************************************
//*Function: IsPhoneBookLoaded()
//*
//*	Parameters:
//*		None.
//*	Returns:
//*		BOOL			- The value of the BOOL variable
//*						  fPhoneBookLoaded.
//*
//******************************************************************
//*

BOOL IsPhoneBookLoaded()	{
	return fPhoneBookLoaded;
}

//*
//******************************************************************
//*Function: SetPhoneBookLoaded()
//*
//*	Parameters:
//*		BOOL			- BOOL value to set the variable
//*						  fPhoneBookLoaded to.
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*

void SetPhoneBookLoaded( BOOL isLoaded )	{
	fPhoneBookLoaded = isLoaded;
}

//*
//******************************************************************
//*Function: GetMaxListViewItems()
//*
//*	Parameters:
//*		None.
//*
//*	Returns:
//*		INT			- Maximum items
//*
//******************************************************************
//*

INT GetMaxListViewItems( void )	{
	return MAX_LISTVIEW_ITEMS;
}

//*
//******************************************************************
//*Function: AddItem() --- not in use
//*
//*	Parameters:
//*
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*

INT AddItem( HWND hwndCtl, INT nItem, LPTSTR pszNumber, LPTSTR pszTitle,
			LPTSTR pszLastCalled, LPTSTR pszLastStatus, INT nSize )	{

	LVITEM lvi;
//	TCHAR	szTmp[40];

	lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
	lvi.iItem = nItem;
	lvi.iSubItem = 0;
	lvi.pszText = pszNumber;
	lvi.iImage = 0;
	lvi.lParam = nItem;
	SendMessage( hwndCtl, LVM_INSERTITEM, 0, (LPARAM)&lvi );

	lvi.mask = LVIF_TEXT;
	lvi.iItem = nItem;
	lvi.iSubItem = 1;
	lvi.pszText = pszTitle;
	SendMessage( hwndCtl, LVM_SETITEM, 0, (LPARAM)&lvi );

	lvi.mask = LVIF_TEXT;
	lvi.iItem = nItem;
	lvi.iSubItem = 2;
	lvi.pszText = pszLastCalled;
	SendMessage( hwndCtl, LVM_SETITEM, 0, (LPARAM)&lvi );

	lvi.mask = LVIF_TEXT;
	lvi.iItem = nItem;
	lvi.iSubItem = 3;
	lvi.pszText = pszLastStatus;
	SendMessage( hwndCtl, LVM_SETITEM, 0, (LPARAM)&lvi );

	return 0;
}
//*
//******************************************************************
//* Function: GetItemData()
//*
//* This function allows fetching of a database record.
//*
//* Parameters:
//*		INT					- Index to ListView element
//*
//*	Returns:
//*		PLVDATAITEM			- ListView DATAITEM structure
//**
//******************************************************************
//*

PLVDATAITEM GetItemData( INT nItem )	{

	INT				nCacheIndex;
	PLVDATAITEM		pdi;

	if ( nItem < TOPCACHESIZE )	{

		nCacheIndex = nItem;
		pdi = &lvdiTopCache[nCacheIndex];
	}

	else if ( nItem > MAX_LISTVIEW_ITEMS - BOTCACHESIZE )	{

		nCacheIndex = nItem - ( MAX_LISTVIEW_ITEMS - BOTCACHESIZE );
		pdi = &lvdiBotCache[nCacheIndex];
	}
	else if (( nItem >= nCacheItemStart ) &&
		( nItem < nCacheItemStart + nCacheSize ))	{

		nCacheIndex = nItem - nCacheItemStart;
		pdi = &lvdiCache[nCacheIndex];
	}
	else
		pdi = GetDatabaseItem( nItem );

	return pdi;
}

//*
//******************************************************************
//*
//* Function: LoadACache()
//*
//* This function will load a database item into cache.
//*
//* Parameters:
//*		PLVDATAITEM			- ListView Data Structure
//*		INT					- Index
//*		INT					- Index
//*
//*	Returns:
//*		INT
//******************************************************************
//*

INT LoadACache( PLVDATAITEM pCache, INT nStart, INT nSize )	{

	PLVDATAITEM		pdi;
	INT				i;

	for( i = 0; i < nSize; i++ )	{

		pdi = GetDatabaseItem( nStart + i );

		// Save the data in the cache

		lstrcpy( pCache[i].Number, pdi->Number );
		lstrcpy( pCache[i].Title, pdi->Title );
		lstrcpy( pCache[i].LastCalled, pdi->LastCalled );
		lstrcpy( pCache[i].LastStatus, pdi->LastStatus );

		pCache[i].nSize = pdi->nSize;
		pCache[i].nImage = pdi->nImage;
		pCache[i].nState = pdi->nState;
	}
	return 0;
}

//*
//******************************************************************
//* Function: LoadMainCache()
//*
//* This function sets up the main cache...
//*
//*	Parameters:
//*		INT					- Index
//*		INT					- Index
//*
//*	Returns:
//*		INT
//*
//******************************************************************
//*

INT LoadMainCache( INT nStart, INT nEnd )	{

	INT		nOverlap;

	if ( nEnd - nStart > CACHESIZE )
		nEnd = nStart + CACHESIZE;

	if ( nEnd > MAX_LISTVIEW_ITEMS - BOTCACHESIZE )	{

		if ( nStart > MAX_LISTVIEW_ITEMS - BOTCACHESIZE )
			return 0;

		nOverlap = nEnd - ( MAX_LISTVIEW_ITEMS - BOTCACHESIZE );
		nEnd = MAX_LISTVIEW_ITEMS - BOTCACHESIZE - 1;

		if ( nStart - nOverlap < TOPCACHESIZE )
			nStart = TOPCACHESIZE;
		else
			nStart -= nOverlap;
	}
	if ( nStart < TOPCACHESIZE )	{

		if ( nEnd < TOPCACHESIZE )
			return 0;

		nOverlap = TOPCACHESIZE - nStart;
		nStart = TOPCACHESIZE;
		if ( nOverlap + nEnd > ( MAX_LISTVIEW_ITEMS - BOTCACHESIZE ))
			nEnd = MAX_LISTVIEW_ITEMS - BOTCACHESIZE;
		else
			nEnd += nOverlap;

	}

	if (( nStart >= nCacheItemStart ) &&
		( nEnd < nCacheItemStart + nCacheSize ))

		return 0;

	FlushMainCache();

	nCacheSize = nEnd - nStart;
	nCacheItemStart = nStart;
	LoadACache( lvdiCache, nStart, nCacheSize );
	return 0;

}

//*
//******************************************************************
//* Function: LoadTopCache()
//*
//* This function loads up the top of the cache...
//*
//*	Parameters:
//*		None.
//*
//*	Returns:
//*		INT
//******************************************************************
//*

INT LoadTopCache( void )	{
	LoadACache( lvdiTopCache, 0, TOPCACHESIZE );
	return 0;
}

//*
//******************************************************************
//* Function: LoadBotCache()
//*
//* This function loads up the bottom of the cache...
//*
//* Parameters:
//*		None.
//*
//*	Returns:
//*		INT
//******************************************************************
//*

INT LoadBotCache( void )	{
	LoadACache( lvdiBotCache, MAX_LISTVIEW_ITEMS - BOTCACHESIZE, BOTCACHESIZE );
	return 0;
}

//*
//******************************************************************
//* Function: FlushMainCache()
//*
//* This function flushes the main cache...
//*
//* Parameters:
//*		None.
//*
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*

void FlushMainCache( void )	{
	
	INT		i;

	for( i = 0; i < nCacheSize; i++ )	{
		SetDatabaseItem( nCacheItemStart + i, &lvdiCache[i] );
	}
	return;
}

//*
//******************************************************************
//* Function: FlushEndCaches()
//*
//* This function flushes top and bottom caches...
//*
//* Parameters:
//*		None.
//*
//*	Returns:
//*		Nothing.
//*
//*
//******************************************************************
//*

void FlushEndCaches( void )	{

	INT		i;

	for( i = 0; i < TOPCACHESIZE; i++ )	{
		SetDatabaseItem( i, &lvdiCache[i] );
	}

	for( i = 0; i < BOTCACHESIZE; i++ )	{
		SetDatabaseItem( MAX_LISTVIEW_ITEMS - BOTCACHESIZE + i, &lvdiBotCache[i] );
	}
	return;
}

//*
//******************************************************************
//* Function: InitDatabaseItem()
//*
//* This function loads the Dial Directory list with an item.
//*
//* Parameters:
//*		INT					- Index
//*		LPTSTR				- String Pointer
//*		LPTSTR				- String Pointer
//*		LPTSTR				- String Pointer
//*		LPTSTR				- String Pointer
//*		INT
//*
//*	Returns:
//*		INT
//*
//******************************************************************
//*

INT InitDatabaseItem( INT nItem, LPTSTR pszNumber, LPTSTR pszTitle,
					 LPTSTR pszLastCalled, LPTSTR pszLastStatus, INT nSize )	{

	lstrcpy( lvDialDirectory[nItem].Number, pszNumber );
	lstrcpy( lvDialDirectory[nItem].Title, pszTitle );
	lstrcpy( lvDialDirectory[nItem].LastCalled, pszLastCalled );
	lstrcpy( lvDialDirectory[nItem].LastStatus, pszLastStatus );

	lvDialDirectory[nItem].nSize = nSize;
	lvDialDirectory[nItem].nImage = 0;
	lvDialDirectory[nItem].nState = 0;
	return 0;
}

//*
//******************************************************************
//* Function: InitDatabaseSubItem()
//*
//* This function loads a database subitem with data.
//*
//* Parameters:
//*		INT					- Item Index
//*		INT					- tag
//*		LPTSTR				- String Pointer
//*
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*

void InitDataBaseSubItem( INT nItem, INT nSubItem, LPTSTR pszData )	{


	PLVDATAITEM		pdi;

	pdi = GetDatabaseItem( nItem );

	switch( nSubItem )	{

		case	1:
			break;

		case	2:

			lstrcpy( pdi->LastCalled, pszData );
			SetDatabaseItem( nItem, pdi );

//			lstrcpy( lvDialDirectory[nItem].LastCalled, pszData );
//			lstrcpy( lvdiCache[nItem].LastCalled,  pszData );
			break;

		case	3:

//			lstrcpy( lvDialDirectory[nItem].LastStatus, pszData );
//			lstrcpy( lvdiCache[nItem].LastStatus,  pszData );
			lstrcpy( pdi->LastStatus, pszData );
			SetDatabaseItem( nItem, pdi );

			break;
	}
//	LoadACache( lvdiCache, nItem, TOPCACHESIZE );
}

//*
//******************************************************************
//*
//* Function: InitDatabase()
//*
//* This function sets up an initial empty database structure
//* if one hasn't been initialized yet.
//*
//* Parameters:
//*		None.
//*
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*

void InitDatabase( void )	{

	TCHAR	szNumber[20];
	TCHAR	szTitle[40];
	TCHAR	szLastCalled[21] = TEXT("");
	TCHAR	szLastStatus[24] = TEXT("");
//	HCURSOR	hOldCur;
	INT		i;

//	hOldCur = SetCursor( LoadCursor( NULL, IDC_WAIT ));

	lstrcpy( szNumber, TEXT( "" ));

	lstrcpy( szTitle, TEXT( "Unused" ));

	if ( !IsPhoneBookLoaded() )	{
		for ( i = 0; i < GetMaxListViewItems(); i++ )	{
			InitDatabaseItem( i, szNumber, szTitle, szLastCalled, szLastStatus, i + 1 );
		}
	}

	gnTotalPhoneDirRecords = GetMaxListViewItems();
//	SetCursor( hOldCur );
	return;
}

//*
//******************************************************************
//*	Function: GetDatabaseItem()
//*
//*		Returns a ListView Item from the database...
//*
//* Parameters:
//*		INT					- Index
//*
//*	Returns:
//*		PLVDATAITEM
//*
//******************************************************************
//*

PLVDATAITEM GetDatabaseItem( INT nItem )	{

	return &lvDialDirectory[nItem];
}

//*
//******************************************************************
//*	Function: SetDatabaseItem()
//*
//*	Parameters:
//*		INT				- Index for database.
//*		PLVDATAITEM	
//*
//*	Returns:
//*		INT
//*
//******************************************************************
//*

INT SetDatabaseItem( INT nItem, PLVDATAITEM pIn )	{

	lstrcpy( lvDialDirectory[nItem].Number, pIn->Number );
	lstrcpy( lvDialDirectory[nItem].Title, pIn->Title );
	lstrcpy( lvDialDirectory[nItem].LastStatus, pIn->LastStatus );
	lstrcpy( lvDialDirectory[nItem].LastCalled, pIn->LastCalled );

	lvDialDirectory[nItem].nSize = pIn->nSize;
	lvDialDirectory[nItem].nImage = pIn->nImage;
	lvDialDirectory[nItem].nState = pIn->nState;
	return 0;
}

//*
//*	End of Database.c
//*