//*
//******************************************************************
//* Program:		Init.c
//*
//* This program contains initialization functions used on start
//* up of WarDialer.
//*
//* Written by:			Byron D Copeland
//*
//******************************************************************
//*

#include <windows.h>
#include "Globals.h"
#include "Resource.h"
#include "UserPref.h"
#include "TTYInfo.h"

// Function prototypes we use in this module...

int InitCommunicationProperties( HWND, HANDLE );
HANDLE InitCommunication( HWND, LPTSTR );
void StartThreads( HWND );
void InitNewFont( LOGFONT LogFont, COLORREF rgbColor );
void InitTTYParameters( void );

// Other external function prototypes..

INT LogResults( HWND, LPTSTR );
DWORD WINAPI SendDataThread( PVOID );
DWORD WINAPI SendThread( PVOID );
DWORD WINAPI ReadThread( PVOID );

//******************************************************************
//* InitCommunicationProperties
//*
//* See what capabilities the com port has....
//******************************************************************

int InitCommunicationProperties( HWND hWnd, HANDLE hComPort )	{

	TCHAR			szTemp[50] = TEXT( "" );

	if ( !SetupComm( COMDEV( TTYInfo ), MAX_READ_BUFFER, MAX_WRITE_BUFFER ))	{

		lstrcpy( szTemp, 
			"InitCommunicationsProperties(): Cannot set Communications Properties.\r\n" );

		SendDlgItemMessage( hWnd, IDC_EDIT_STATUSMSGS, EM_REPLACESEL,
						0, (LPARAM)szTemp );
		LogResults( hWnd, szTemp );
	}

	return 0;
}

//******************************************************************
//* InitCommunication
//*
//* Initiate communications with a port....
//******************************************************************

HANDLE InitCommunication( HWND hWnd, LPTSTR pszDevName )	{

	DCB				dcb;
	INT				i;
	TCHAR			szDbg[128];
	COMMTIMEOUTS	cto;

	i = lstrlen( pszDevName );

	COMDEV( TTYInfo ) = CreateFile( &pszDevName[i - 5], GENERIC_READ | GENERIC_WRITE,
						0, 
						0, 
						OPEN_EXISTING, 
                        FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
						0 );


	if ( COMDEV( TTYInfo ) != INVALID_HANDLE_VALUE )	{


		if (!GetCommTimeouts( COMDEV( TTYInfo ), &(TIMEOUTSORIG(TTYInfo))))	{

			lstrcpy( szDbg, TEXT("Error: InitializeCommunication() GetCommTimeouts()\r\n" ));

			SendDlgItemMessage( hWnd, IDC_JJ_RESULTS, EM_REPLACESEL,
						0, (LPARAM)szDbg );

		LogResults( hWnd, szDbg );

//			ErrorReporter("GetCommTimeouts");
		}

		// configure the port

		dcb.DCBlength = sizeof( &dcb );

		GetCommState( COMDEV( TTYInfo ), &dcb );

		// set up the dcb like we want it now.
		
		dcb.BaudRate = userSavedSettings.dcb.BaudRate;
		dcb.ByteSize = userSavedSettings.dcb.ByteSize;
		dcb.fParity = userSavedSettings.dcb.fParity;
		dcb.fNull = userSavedSettings.dcb.fNull;
		dcb.StopBits = userSavedSettings.dcb.StopBits;
		dcb.Parity = userSavedSettings.dcb.Parity;

		dcb.fBinary = userSavedSettings.dcb.fBinary;
		dcb.fOutxCtsFlow = userSavedSettings.dcb.fOutxCtsFlow;
		dcb.fOutxDsrFlow = userSavedSettings.dcb.fOutxDsrFlow;
		dcb.fDsrSensitivity = userSavedSettings.dcb.fDsrSensitivity;
		dcb.fDtrControl = userSavedSettings.dcb.fDtrControl;
		dcb.fOutX = userSavedSettings.dcb.fOutX;
		dcb.fInX = userSavedSettings.dcb.fInX;
		dcb.fErrorChar = userSavedSettings.dcb.fErrorChar;
		dcb.fRtsControl = userSavedSettings.dcb.fRtsControl;
		dcb.fAbortOnError = userSavedSettings.dcb.fAbortOnError;


		SetCommState( COMDEV( TTYInfo ), &dcb );

		// Timeouts...

		cto.ReadIntervalTimeout = 0x01;
		cto.ReadTotalTimeoutMultiplier = 1000;
		cto.ReadTotalTimeoutConstant = 0;
		cto.WriteTotalTimeoutMultiplier = 0;
		cto.WriteTotalTimeoutConstant = 0;

		SetCommTimeouts( COMDEV( TTYInfo ), &cto );

		EscapeCommFunction( COMDEV( TTYInfo ), SETDTR );
		EscapeCommFunction( COMDEV( TTYInfo ), SETRTS );

		// Save these params for later when user wants
		// store these as defaults...

//		userSavedSettings.dcb = dcb;
		userSavedSettings.cto = cto;
//		lstrcpy( userSavedSettings.szPortInUse, pszDevName );

		wsprintf( szDbg, TEXT( "Port %s is Ready!\r\n" ), pszDevName );

//		SendDlgItemMessage( hWnd, ID_RCVTEXT, EM_REPLACESEL, 0,
//							(LPARAM)szDbg );

		SendDlgItemMessage( hWnd, IDC_EDIT_STATUSMSGS, EM_REPLACESEL,
								0, (LPARAM)szDbg );

		LogResults( hWnd, szDbg );
		
		if ( *pszDevName == TEXT( 'R' ))	{
//			if ( !EscapeCommFunction( hLocal, SETIR ))	{
//				wsprintf( szDbg, TEXT( "Set IR Failed. rc %d\r\n" ),
//					GetLastError());
//				SendDlgItemMessage( hWnd, ID_RCVTEXT, EM_REPLACESEL,
//									0, (LPARAM)szDbg );

				SendDlgItemMessage( hWnd, IDC_EDIT_STATUSMSGS, EM_REPLACESEL,
								0, (LPARAM)szDbg );

//			}
		}

		CONNECTED( TTYInfo )     = FALSE ;
		LOCALECHO( TTYInfo )     = FALSE ;
		CURSORSTATE( TTYInfo )   = CS_HIDE ;
		PORT( TTYInfo )          = 1 ;
		BAUDRATE( TTYInfo )      = userSavedSettings.dcb.BaudRate;
		BYTESIZE( TTYInfo )      = 8 ;
		PARITY( TTYInfo )        = NOPARITY ;
		STOPBITS( TTYInfo )      = ONESTOPBIT ;
//		AUTOWRAP( TTYInfo )      = TRUE;
//		NEWLINE( TTYInfo )       = FALSE;
//		XSIZE( TTYInfo )         = 0 ;
//		YSIZE( TTYInfo )         = 0 ;
//		XSCROLL( TTYInfo )       = 0 ;
//		YSCROLL( TTYInfo )       = 0 ;
//		COLUMN( TTYInfo )        = 0 ;

		DISPLAYERRORS( TTYInfo ) = TRUE ;

    //
    // timeouts
    //

		TIMEOUTSNEW( TTYInfo )   = cto;

    //
    // read state and status events
    //
		gdwReceiveState				= RECEIVE_TTY;
		EVENTFLAGS( TTYInfo )		= EVENTFLAGS_DEFAULT;
		FLAGCHAR( TTYInfo )			= FLAGCHAR_DEFAULT;

    //
    // Flow Control Settings
    //
		DTRCONTROL( TTYInfo )		= DTR_CONTROL_ENABLE;
		RTSCONTROL( TTYInfo )		= RTS_CONTROL_ENABLE;
		XONCHAR( TTYInfo )			= ASCII_XON;
		XOFFCHAR( TTYInfo )			= ASCII_XOFF;
		XONLIMIT( TTYInfo )			= 0;
		XOFFLIMIT( TTYInfo )		= 0;
		CTSOUTFLOW( TTYInfo )		= FALSE;
		DSROUTFLOW( TTYInfo )		= FALSE;
		DSRINFLOW( TTYInfo )		= FALSE;
		XONXOFFOUTFLOW(TTYInfo)		= FALSE;
		XONXOFFINFLOW(TTYInfo)		= FALSE;
		TXAFTERXOFFSENT(TTYInfo)	= FALSE;

		NOREADING(TTYInfo)			= FALSE;
		NOWRITING(TTYInfo)			= FALSE;
		NOEVENTS(TTYInfo)			= FALSE;
		NOSTATUS(TTYInfo)			= FALSE;
		SHOWTIMEOUTS(TTYInfo)		= FALSE;
		VOICECAPABLE( TTYInfo )		= FALSE;
		FAXCAPABLE( TTYInfo )		= FALSE;
		VOICEVIEWCAPABLE( TTYInfo ) = FALSE;
//		TTYWINDOWISOPEN( TTYInfo )  = FALSE;

    //
    // setup default font information
    // 
	
	
		StartThreads( hWnd);

		CONNECTED( TTYInfo ) = TRUE;

		return COMDEV( TTYInfo );

	} else	{


		wsprintf( szDbg, TEXT( "Problem opening Com Port %s\r\n%s\r\n" ),
				pszDevName, "" );

		SendDlgItemMessage( hWnd, IDC_EDIT_STATUSMSGS, EM_REPLACESEL,
								0, (LPARAM)szDbg );
		LogResults( hWnd, szDbg );

		MessageBeep( MB_ICONEXCLAMATION );
	}


	return NULL;
}

void InitTTYParameters( void )	{

	gwBaseY = HIWORD( GetDialogBaseUnits() );

//	cyMenuHieght = GetSystemMetrics( SM_CYMENU );
//	cyCaptionHeight = GetSystemMetrics( SM_CYCAPTION );
//	cyFrameHeight = GetSystemMetrics( SM_CYFRAME );

//	gcyMinimumWindowHeight = cyMenuHeight + 4 * cyCaptionHeight +
//								2 * cyFrameHeight +
//								( SETTINGFACTOR + STATUSFACTOR ) * gwBaseY;

	AUTOWRAP( TTYInfo )      = TRUE;
	NEWLINE( TTYInfo )       = FALSE;
	XSIZE( TTYInfo )         = 0;
	YSIZE( TTYInfo )         = 0;
	XSCROLL( TTYInfo )       = 0 ;
	YSCROLL( TTYInfo )       = 0 ;
	COLUMN( TTYInfo )        = 0 ;

	ROW( TTYInfo )           = MAXROWS - 1 ;
	LFTTYFONT( TTYInfo ).lfHeight =         12 ;
	LFTTYFONT( TTYInfo ).lfWidth =          0 ;
	LFTTYFONT( TTYInfo ).lfEscapement =     0 ;
	LFTTYFONT( TTYInfo ).lfOrientation =    0 ;
	LFTTYFONT( TTYInfo ).lfWeight =         0 ;
	LFTTYFONT( TTYInfo ).lfItalic =         0 ;
	LFTTYFONT( TTYInfo ).lfUnderline =      0 ;
	LFTTYFONT( TTYInfo ).lfStrikeOut =      0 ;
	LFTTYFONT( TTYInfo ).lfCharSet =        OEM_CHARSET ;
	LFTTYFONT( TTYInfo ).lfOutPrecision =   OUT_DEFAULT_PRECIS ;
	LFTTYFONT( TTYInfo ).lfClipPrecision =  CLIP_DEFAULT_PRECIS ;
	LFTTYFONT( TTYInfo ).lfQuality =        DEFAULT_QUALITY ;
	LFTTYFONT( TTYInfo ).lfPitchAndFamily = FIXED_PITCH | FF_MODERN ;
	strcpy( LFTTYFONT( TTYInfo ).lfFaceName, "FixedSys" ) ;

	InitNewFont( LFTTYFONT( TTYInfo ), RGB( 0, 0, 0 ));


}

//*
//******************************************************************
//*
//******************************************************************
//*

void InitNewFont( LOGFONT LogFont, COLORREF rgbColor )	{

	TEXTMETRIC	tm;
	HDC			hDC;

	if ( HTTYFONT( TTYInfo ))
		DeleteObject( HTTYFONT( TTYInfo ));

	LFTTYFONT( TTYInfo ) = LogFont;
	HTTYFONT( TTYInfo ) = CreateFontIndirect( &(LFTTYFONT( TTYInfo )));
	FGCOLOR( TTYInfo ) = rgbColor;

	hDC = GetDC( ghWndMain );
	SelectObject( hDC, HTTYFONT( TTYInfo ));
	GetTextMetrics( hDC, &tm );
	ReleaseDC( ghWndMain, hDC );

	XCHAR( TTYInfo ) = tm.tmAveCharWidth;
	YCHAR( TTYInfo ) = tm.tmHeight + tm.tmExternalLeading;

	XOFFSET( TTYInfo ) = 0;
	YOFFSET( TTYInfo ) = YCHAR( TTYInfo ) * ROW( TTYInfo );

}

void StartThreads( HWND hWnd )	{

    DWORD dwReadStatId;
    DWORD dwWriterId;
	DWORD dwSendId;
	TCHAR szErr[50];

    SENDDATATHREAD(TTYInfo) = 
            CreateThread( NULL, 
                          0,
                          (LPTHREAD_START_ROUTINE) SendDataThread,
//                          ReadThread,
                          (LPVOID) hWnd, 
                          0, 
                          &dwSendId);

    if (SENDDATATHREAD(TTYInfo) == NULL)	{

		lstrcpy( szErr, TEXT("Error: CreateThread(Send)\r\n" ));

		SendDlgItemMessage( hWnd, IDC_JJ_RESULTS, EM_REPLACESEL,
								0, (LPARAM)szErr );
		LogResults( hWnd, szErr );
		return;
//        ErrorInComm("CreateThread(Reader/Status)");
	}
	lstrcpy( szErr, TEXT("SendDataThread Started.\r\n" ));

	SendDlgItemMessage( hWnd, IDC_EDIT_STATUSMSGS, EM_REPLACESEL,
								0, (LPARAM)szErr );

    READSTATTHREAD(TTYInfo) = 
            CreateThread( NULL, 
                          0,
                          (LPTHREAD_START_ROUTINE) ReadThread,
//                          ReadThread,
                          (LPVOID) hWnd, 
                          0, 
                          &dwReadStatId);

    if (READSTATTHREAD(TTYInfo) == NULL)	{

		lstrcpy( szErr, TEXT("Error: CreateThread(Reader/Status)\r\n" ));

		SendDlgItemMessage( hWnd, IDC_JJ_RESULTS, EM_REPLACESEL,
								0, (LPARAM)szErr );
		LogResults( hWnd, szErr );
		return;
//        ErrorInComm("CreateThread(Reader/Status)");
	}

	lstrcpy( szErr, TEXT("ReadThread Started.\r\n" ));

	SendDlgItemMessage( hWnd, IDC_EDIT_STATUSMSGS, EM_REPLACESEL,
								0, (LPARAM)szErr );
	LogResults( hWnd, szErr );

    WRITERTHREAD(TTYInfo) = 
            CreateThread( NULL, 
                          0, 
                          (LPTHREAD_START_ROUTINE) SendThread, 
//                          SendThread, 
                          (LPVOID) NULL, 
                          0, 
                          &dwWriterId );
                   
    if (WRITERTHREAD(TTYInfo) == NULL)	{
		lstrcpy( szErr, TEXT("Error: CreateThread(Reader/Status)\r\n" ));

		SendDlgItemMessage( hWnd, IDC_JJ_RESULTS, EM_REPLACESEL,
								0, (LPARAM)szErr );

		LogResults( hWnd, szErr );
//        ErrorInComm("CreateThread (Writer)");
		return;
	}

	lstrcpy( szErr, TEXT("WriteThread Started.\r\n" ));

	SendDlgItemMessage( hWnd, IDC_EDIT_STATUSMSGS, EM_REPLACESEL,
								0, (LPARAM)szErr );
	LogResults( hWnd, szErr );

    return;
}


//
//
//


void GlobalInitialize( HWND hWnd )	{

//    int cyMenuHeight, cyCaptionHeight, cyFrameHeight;

	SYSTEM_INFO		SysInfo;

	GetSystemInfo( &SysInfo );
    //
    // critical sections in status reporting & node management
    //
    InitializeCriticalSection(&gStatusCritical);
    InitializeCriticalSection(&gcsWriterHeap);
    InitializeCriticalSection(&gcsDataHeap);

    //
    // status message event
    //
    ghStatusMessageEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (ghStatusMessageEvent == NULL)
		MessageBox( hWnd, "Error: CreateEvent() (Status message event)", "Global Init", 0 );

//        ErrorReporter("CreateEvent (Status message event)");

    //
    // thread exit event
    //
    ghThreadExitEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

    if (ghThreadExitEvent == NULL)
		MessageBox( hWnd, "Error: CreateEvent() (Thread exit event)",
				"GlobalInitialize", 0 );

//        ErrorReporter("CreateEvent (Thread exit event)");        

    ghStatusMessageHeap = HeapCreate(0, SysInfo.dwPageSize, 0);
    if (ghStatusMessageHeap == NULL)
		MessageBox( hWnd, "Error: HeapCreate ( ghStatusMessageHeap)",
				"GlobalInitialize", 0 );

//        ErrorReporter("HeapCreate (Status message)");

    glpStatusMessageHead = HeapAlloc(ghStatusMessageHeap, HEAP_ZERO_MEMORY, sizeof(STATUS_MESSAGE));
    glpStatusMessageTail = glpStatusMessageHead;

    gnStatusIndex = 0;

    //
    // used in file transfer status bar
    //
//    InitCommonControls();

    //
    // font for status reporting control
    //
 //   ghFontStatus = CreateStatusEditFont();

    //
    // the following are used for sizing the tty window and dialog windows
    //
//    gwBaseY = HIWORD(GetDialogBaseUnits());
//    cyMenuHeight = GetSystemMetrics(SM_CYMENU);
//    cyCaptionHeight = GetSystemMetrics(SM_CYCAPTION);
//    cyFrameHeight = GetSystemMetrics(SM_CYFRAME);
//    gcyMinimumWindowHeight = cyMenuHeight + \
//                            4 * cyCaptionHeight + \
//                            2 * cyFrameHeight +
//                            (SETTINGSFACTOR + STATUSFACTOR) * gwBaseY ;

	szTextOut = (char *)malloc( 1024 * sizeof( char ));
    return;
}

void GlobalCleanup()
{
    DeleteCriticalSection(&gStatusCritical);
    DeleteCriticalSection(&gcsWriterHeap);
    DeleteCriticalSection(&gcsDataHeap);
//    DeleteObject(ghFontStatus);
    CloseHandle(ghStatusMessageEvent);
    CloseHandle(ghThreadExitEvent);
    HeapDestroy(ghStatusMessageHeap);

	if ( szTextOut )
		free( szTextOut );
    return;
}
