//*
//******************************************************************
//* Header:		Kalimba.h
//*
//* This header file contains all defines, structures,
//* macros, variables for the main program Kalimba.
//*
//* Written by:			Byron D Copeland
//*
//******************************************************************
//*

#include "Resource.h"
#include "TTYInfo.h"
#include "Database.h"

#define dim(x) (sizeof(x) / sizeof(x[0]))

// For Dialing Directory Defines
//#define MAX_LISTVIEW_ITEMS			2048
//#define CACHESIZE		100
//#define TOPCACHESIZE	100
//#define BOTCACHESIZE	100

//
// window coords
//
#define MAXXWINDOW          750
#define MAXYWINDOW          530
#define STARTXWINDOW        80
#define STARTYWINDOW        70

//#define SETTINGSFACTOR      5
//#define STATUSFACTOR        100
//
// window timer ids
// 
#define TIMERID					1
#define JJ_READ_TIMERID			100
#define JJ_VOICE_TIMERID		110
#define JJ_STARTUP_TIMERID		120
#define JJ_WAIT_5SECS_TIMERID	130
#define JJ_STALL_WAIT_TIMER		135
#define JJ_ANSWERCALL_TIMERID	140

struct decodeUINT	{
	UINT	Code;

	LRESULT (*Fxn)(HWND, UINT, WPARAM, LPARAM );

};

struct decodeCMD	{
	UINT Code;

	LRESULT (*Fxn)(HWND, WORD, HWND, WORD );
};


char		lpBuf[AMOUNT_TO_READ];
char		szTextToSend[1024];

// Voice Command Responses

char *ShieldedDLE				= "\0x10\0";	// Data Link Escape (DLE)
char *ShieldedETX				= "\0x03\0";	// End of voice xmt (ETX)
char *ShieldedAnswerTone		= "a\0";		// Data/Fax 2100 Hz
char *ShieldedBusyTone			= "b\0";		// Doot-Doot-Doot-Doot
char *ShieldedFaxCallTone		= "c\0";		//
char *ShieldedDialTone			= "d\0";		//
char *ShieldedDataCallTone		= "e\0";		// Bell South 2225 Hz
char *ShieldedLocalOnHook		= "h\0";		// On Hook
char *ShieldedLocalOffHook		= "H\0";		// Off Hook
char *ShieldedRing				= "R\0";		// Phone's Ringing
char *ShieldedSilenceTimer		= "s\0";		// Expired
char *ShieldedCASTone			= "@\0";		// Got CAS tones

char *FCLASSDataMode			= "AT+FCLASS=0\0";	// Switch to Data mode
char *FCLASSFaxMode				= "AT+FCLASS=1\0";	// Switch to FAX mode
char *FCLASSVoiceMode			= "AT+FCLASS=8\0";	// Switch to voice mode
char *FCLASSVoiceView			= "AT+FCLASS=80\0"; // Switch to VoiceView mode
char *FCLASSSpeakerON			= "AT+VLS=7\0";		// Turn speaker on and
													// go OFFHOOK.


unsigned long uNextRnd = 1;

int uNextRndSeq = 1;
//unsigned int Seqs[MAX_LISTVIEW_ITEMS], Seqj, Seqi, SeqTmp, Seqx;

BOOL		fFaxDetectMode				= FALSE;
BOOL		fInVoiceMode				= FALSE;
BOOL		fVoiceDetectMode			= FALSE;
BOOL		fInDataMode					= TRUE;		// Default mode
BOOL		fInFaxMode					= FALSE;
BOOL		fInAnswerMode				= FALSE;
BOOL		fReadNotDone				= FALSE;
BOOL		fWritePending				= FALSE;
BOOL		fRunningScan				= FALSE;

BOOL		fWaitingOnRead				= FALSE;
BOOL		fWaitingOnStat				= FALSE;
BOOL		fFirstTime					= TRUE;

BOOL		fNewMaskLoaded				= FALSE;
BOOL		fWaitingForResponse			= FALSE;
//BOOL		fItemOneLoaded				= FALSE;
BOOL		fConnectionEstablished		= FALSE;
BOOL		fStartupTimersAgain			= FALSE;
BOOL		fUpdateStatusNow			= TRUE;
BOOL		fAnsweringCall				= FALSE;

// switch to toggle display over to a TTY screen

BOOL		fSendDataToTTYScreen = FALSE;

//BOOL		fAlreadyDialed[MAX_LISTVIEW_ITEMS];

//DWORD		nCurrentLVdi;
UINT		uDelayTimerValue = 0;
UINT		uNextRecordToCall = 0;
UINT		uTotalRecordsInCall = 0;
UINT		uStartupTimer = 0;
UINT		uTimerValue = 0;
UINT		uNewRandTime = 0;
UINT		uWait5SecondsTimer = 0;
UINT		uAnswerCallTimer = 0;

UINT		uStallWaitTimerValue = 1000;
UINT		uStallWaitValue = 0;
UINT		uStallWaitTimer = 0;

//
//
//

typedef struct _kalimbaBlackList	{
	TCHAR			Title[64];
	TCHAR			Number[20];
} KalimbaBL, *lpKalimbaBL;



BOOL gfAbortTransfer;

//
//  Write request data structure;
//

typedef struct WRITEREQUEST;

typedef struct WRITEREQUEST
{
  DWORD      dwWriteType;        // char, file start, file abort, file packet
  DWORD      dwSize;             // size of buffer
  char       ch;                 // ch to send
  char *     lpBuf;              // address of buffer to send
  HANDLE     hHeap;              // heap containing buffer
  HWND       hWndProgress;       // status bar window handle
  struct WRITEREQUEST *pNext;    // next node in the list
  struct WRITEREQUEST *pPrev;    // prev node in the list
} WRITEREQUEST, *PWRITEREQUEST;

struct WRITEREQUEST *gpWriterHead;
struct WRITEREQUEST *gpWriterTail;

//
//  Status updating
//


NMHDR		 rtpnmhdr;
NMLVDISPINFO rtpLVdi;


//
// Kalimba Function Prototypes
//

DWORD WINAPI SendDataThread( PVOID pArg );
//int ReadThread( PVOID pArg );
DWORD WINAPI ReadThread( PVOID pArg );
//int SendThread( PVOID pArg );
DWORD WINAPI SendThread( PVOID pArg );
HANDLE InitCommunication( HWND, LPTSTR );
INT GetIrCommDeviceName( LPTSTR );
INT GetRawIrDeviceName( LPTSTR );
int FillComComboBox( HWND );
//LPTSTR HandleErrorCode( HWND, LPTSTR, LPTSTR );
void GetCurrentDateTime( void );
BOOL BuildNumberMaskList( HWND, char * );
void AddMaskValueToDatabase( INT, PLVDATAITEM );

int InitCommunicationProperties( HWND, HANDLE );

void SetCommsDefaultBasicProperties( HWND );
void SetCommsDefaultInitsProperties( HWND );
void SetCommsDefaultMessageProperties( HWND );
void SetCommsDefaultAdvancedProperties( HWND );
void SetInitialCommunicationsDefaults( void );
void SendModemInitializationStrings( HWND );

int InitApp( HINSTANCE );
HWND InitInstance( HINSTANCE, LPWSTR, int );
int TermInstance( HINSTANCE, int );

//
// Kalimba Window Procedures
//
LRESULT CALLBACK MainWndProc( HWND, UINT, WPARAM, LPARAM );

//
// Our Message Handlers
//
LRESULT DoCreateMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoCharMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoSizeMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoSetFocusMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoCommandMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoNotifyMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoDestroyMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoQueryEndSessionMain( HWND, UINT, WPARAM, LPARAM );
LRESULT DoTermInstance( HWND, UINT, WPARAM, LPARAM );
LRESULT DoSysCommandMain( HWND, UINT, WPARAM, LPARAM );

//
// Kalimba Command Functions
//
LPARAM DoMainCommandExit( HWND, WORD, HWND, WORD );
LPARAM DoMainCommandComPort( HWND, WORD, HWND, WORD );
LPARAM DoMainCommandSendText( HWND, WORD, HWND, WORD );
LPARAM DoMainSelectPhoneBook( HWND, WORD, HWND, WORD );
LPARAM DoMainCommandAbout( HWND, WORD, HWND, WORD );
LPARAM DoMainSerialCommPortSetup( HWND, WORD, HWND, WORD );
LPARAM DoMainLoadUserSettings( HWND, WORD, HWND, WORD );
LPARAM DoMainSaveUserSettings( HWND, WORD, HWND, WORD );
LPARAM DoMainSavePhoneBook( HWND, WORD, HWND, WORD );
LPARAM DoMainImportPhoneFile( HWND, WORD, HWND, WORD );
LPARAM DoMainEditShowDialDir( HWND, WORD, HWND, WORD );
LPARAM DoMainCommandChView( HWND, WORD, HWND, WORD );
LPARAM DoMainWarDialer( HWND, WORD, HWND, WORD );

LPARAM LoadUserSettings( HWND, WORD, HWND, WORD );
LPARAM SaveUserSettings( HWND, WORD, HWND, WORD );
LPARAM SelectPhoneBook( HWND, WORD, HWND, WORD );
LPARAM SavePhoneBook( HWND, WORD, HWND, WORD );
LPARAM ImportPhoneFile( HWND, WORD, HWND, WORD );
LPARAM SerialCommPortSetup( HWND, WORD, HWND, WORD );
LPARAM EditShowDialDir( HWND, WORD, HWND, WORD );
LPARAM ThingsAboutKalimba( HWND, WORD, HWND, WORD );

//BOOL CALLBACK DialStatusWindowProc( HWND, UINT, WPARAM, LPARAM );
BOOL CALLBACK JumpJiveWindowProc( HWND, UINT, WPARAM, LPARAM );

//PB Edit Dialog Proc
void ResetModemDTRSignal( HWND );

LRESULT ListViewNotify( HWND, LPARAM );

BOOL InitListView(HWND);
void InitDataBaseSubItem( INT, INT, LPTSTR );
INT SetDatabaseItem( INT, PLVDATAITEM );
PLVDATAITEM GetDatabaseItem( INT );
INT LoadBotCache( void );
INT LoadTopCache( void );
INT LoadACache( PLVDATAITEM, INT, INT );
void InitDatabase( void );
INT InitDatabaseItem( INT, LPTSTR, LPTSTR, LPTSTR, LPTSTR, INT );
void FlushEndCaches( void );
void FlushMainCache( void );
INT LoadMainCache( INT, INT );
PLVDATAITEM GetItemData( INT );
INT AddItem( HWND, INT, LPTSTR, LPTSTR,	LPTSTR, LPTSTR, INT );
INT GetMaxListViewItems( void );

void CheckReturnedModemResults( HWND );
void ReturnStatusResetTimers( HWND );
void PruneBuffer( void );
int NewRandom( void );
void SetRandom( unsigned int );
void SetRandomSequence( void );

int NewRandomSequence( void );

void StartThreads( HWND );
DWORD WaitForThreads( HWND, DWORD );

void GlobalInitialize( HWND );
void GlobalCleanup( void );
void HandleCommWriteRequests( HWND );
void WriterComplete( HWND );
void WriterAbort( HWND, PWRITEREQUEST );
void WriterFile( HWND, PWRITEREQUEST );
void WriterBlock( HWND, PWRITEREQUEST );
void WriterFileStart( DWORD );
void WriterChar( HWND, PWRITEREQUEST );
void WriterGeneric( HWND, char *, DWORD );
//void CheckModemStatus( void );
void ReportModemStatus( HWND, DWORD );
void CheckModemStatus( HWND, BOOL );
void SetPhoneBookLoaded( BOOL );
BOOL IsPhoneBookLoaded( void );
void OutputABufferToWindow( HWND hTTY, char * lpBuf, DWORD dwBufLen );

// Keypad function prototypes
void OpenNumericKeyPadDialog( HINSTANCE, HWND );

// Prototypes for Reports.c

INT LogResults( HWND, LPTSTR );
INT CreateFreshResultsLog( HWND, LPTSTR );
INT LogEvents( HWND, LPTSTR );
INT CreateFreshEventsLog( HWND, LPTSTR );

//

PWRITEREQUEST RemoveFromLinkedList( PWRITEREQUEST, HWND );
void AddToLinkedList( PWRITEREQUEST, HWND );
void AddToFrontOfLinkedList( PWRITEREQUEST, HWND );
BOOL WriterAddNewNode( DWORD, DWORD, char, char *, HANDLE, HWND, HWND );
BOOL WriterAddNewNodeTimeout( DWORD, DWORD, char, char *, HANDLE, HWND, DWORD,	HWND );
BOOL WriteAddFirstNodeTimeout( DWORD, DWORD, char, char *, HANDLE, HWND, DWORD, HWND );
BOOL WriterAddExistingNode( PWRITEREQUEST, DWORD, DWORD, char, char *, HANDLE, HWND, HWND );
void OpenTTYWindow( HWND hWnd );
void CloseTTYWindow( void );
int WINAPI TTYChildProc( HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam );
void InitTTYParameters( void );
