//*
//******************************************************************
//* Module:		KeyPad.c
//*
//*		This module contains functions that display a dialog
//*	that contains a standard telephone keypad arrangement &
//* special keys for computer access.  The tones will be played
//* through the computers' sound card.  Such as a soundblaster.
//*
//*		The dialog also contains the infamous blue box, red box
//* tone generators.
//*
//*		Standards of tone generation can be selected by the user
//*	to generate DTMF or CCITT5 multi-frequency tones.
//*
//*Author:		Byron D Copeland
//*
//*Date:		30 Aug 99
//*
//******************************************************************
//*
#include <windows.h>
#include <commctrl.h>
#include <mmsystem.h>
//#include "Globals.h"
#include "Resource.h"
#include "TTYInfo.h"
#include "ToneInfo.h"

#define			TONE_DTMF		1
#define			TONE_CCITT		2
#define			TONE_BLUEBOX	3

HWND			hWndKeyPadDlgOwner;
WAVEOUTCAPS		pwoc;
HWAVEOUT		hWaveOutDev;	// Handle to Ouput Device
WAVEHDR			WaveHeader;

HMMIO			hWaveHandle;

DWORD			Res;

BOOL			fInUse = FALSE;
UINT			uToneStandard = TONE_DTMF;
UINT			uWaveItemToPlay;
TCHAR			szTmp[50] = TEXT("");

// Prototypes of what we use here...

void PlayWaveData( HWND, UINT );
void OpenNumericKeyPadDialog( HINSTANCE, HWND );
BOOL CALLBACK KeyPadDialogProc( HWND, UINT, WPARAM, LPARAM );
void CheckWaveError( HWND, DWORD, char * );
void CheckMMIOError( HWND, DWORD );
void BuildMMIOToneItems( HWND );

//*
//******************************************************************
//* Function: OpenNumericKeyPadDialog
//*
//*		Initiate the Key Pad Dialog box.
//*
//* Parameters:
//*		HINSTANCE		- The instance of the main window.
//*		HWND			- Handle to the calling Window/Dialog.
//*
//*	Returns:
//*		Nothing.
//******************************************************************
//*

void OpenNumericKeyPadDialog( HINSTANCE hInst, HWND hWnd )	{

	// Lets save the handle to the window/dialog that called this
	// function.

	hWndKeyPadDlgOwner = hWnd;

	// Display the dialog and hand processes off to the
	// process that will handle its messages.

	CreateDialog( hInst, 
					MAKEINTRESOURCE( IDD_MAN_TONE_DIALOG ), 
					hWnd,
					KeyPadDialogProc );

	return;
}

//*
//******************************************************************
//* Function: KeyPadDialogProc
//*
//*		This function will handle the messaging for the keypad
//*	dialog box...
//*
//*	Parameters:
//*		HWND		- Handle to Window/Dialog using this process.
//*		UINT		- The message passed into this process we will
//*					  process.
//*		WPARAM		- Parameter of type WORD
//*		LPARAM		- Parameter of type LONG
//*					  WPARAM & LPARAM are used to pass special
//*					  arguments allong with the message.  Depending
//*					  on the type message passed, all, one, or none
//*					  of these parameters may contain data.
//*
//*	Returns:
//*		BOOL		- TRUE When the dialog destroys itself, otherwise 
//*					  returns FALSE.
//*
//******************************************************************
//*

BOOL CALLBACK KeyPadDialogProc( HWND hWnd, 
							   UINT wMsg, 
							   WPARAM wParam,
							   LPARAM lParam )	{

	BOOL	fRet = FALSE;
	UINT		i = 0;

	switch( wMsg )	{

		case WM_CHAR:

			switch( (TCHAR)wParam )	{

				case '1':

					MessageBox( hWnd, "Got Char", "WM_CHAR", 0 );
					break;
			}

			return FALSE;

		// The standard message passed to us by the OS whenever 
		// the wave file is done playing...
		// At this point we want to close the output device.

		case MM_WOM_DONE:

			Res = waveOutUnprepareHeader( hWaveOutDev, &WaveHeader,
						sizeof( WAVEHDR ));

			Res = waveOutClose( hWaveOutDev );
			fInUse = FALSE;

			return FALSE;

		// The standard message passed whenever a dialog box
		// is created, but hasn't been displayed yet.
		// This allows us to perform some preliminary setup
		// before the user actually sees the dialog

		case WM_INITDIALOG:

			// We need to load the wave files ahead of time just
			// so we can be ready to send them to the modem

			if ( uToneStandard == TONE_DTMF )	{
				lstrcpy( DTMFWaveItems[0].szWavePath, "C:\\Kalimba\\Audio\\0.wav" );
				lstrcpy( DTMFWaveItems[1].szWavePath, "C:\\Kalimba\\Audio\\1.wav" );
				lstrcpy( DTMFWaveItems[2].szWavePath, "C:\\Kalimba\\Audio\\2.wav" );
				lstrcpy( DTMFWaveItems[3].szWavePath, "C:\\Kalimba\\Audio\\3.wav" );
				lstrcpy( DTMFWaveItems[4].szWavePath, "C:\\Kalimba\\Audio\\4.wav" );
				lstrcpy( DTMFWaveItems[5].szWavePath, "C:\\Kalimba\\Audio\\5.wav" );
				lstrcpy( DTMFWaveItems[6].szWavePath, "C:\\Kalimba\\Audio\\6.wav" );
				lstrcpy( DTMFWaveItems[7].szWavePath, "C:\\Kalimba\\Audio\\7.wav" );
				lstrcpy( DTMFWaveItems[8].szWavePath, "C:\\Kalimba\\Audio\\8.wav" );
				lstrcpy( DTMFWaveItems[9].szWavePath, "C:\\Kalimba\\Audio\\9.wav" );
			}
			if ( uToneStandard == TONE_CCITT )	{
				lstrcpy( DTMFWaveItems[0].szWavePath, "C:\\Kalimba\\Audio\\0ccitt.wav" );
				lstrcpy( DTMFWaveItems[1].szWavePath, "C:\\Kalimba\\Audio\\1ccitt.wav" );
				lstrcpy( DTMFWaveItems[2].szWavePath, "C:\\Kalimba\\Audio\\2ccitt.wav" );
				lstrcpy( DTMFWaveItems[3].szWavePath, "C:\\Kalimba\\Audio\\3ccitt.wav" );
				lstrcpy( DTMFWaveItems[4].szWavePath, "C:\\Kalimba\\Audio\\4ccitt.wav" );
				lstrcpy( DTMFWaveItems[5].szWavePath, "C:\\Kalimba\\Audio\\5ccitt.wav" );
				lstrcpy( DTMFWaveItems[6].szWavePath, "C:\\Kalimba\\Audio\\6ccitt.wav" );
				lstrcpy( DTMFWaveItems[7].szWavePath, "C:\\Kalimba\\Audio\\7ccitt.wav" );
				lstrcpy( DTMFWaveItems[8].szWavePath, "C:\\Kalimba\\Audio\\8ccitt.wav" );
				lstrcpy( DTMFWaveItems[9].szWavePath, "C:\\Kalimba\\Audio\\9ccitt.wav" );
			}
			if ( uToneStandard == TONE_BLUEBOX )	{
				lstrcpy( DTMFWaveItems[0].szWavePath, "C:\\Kalimba\\Audio\\bb10.wav" ); // Guess?
				lstrcpy( DTMFWaveItems[1].szWavePath, "C:\\Kalimba\\Audio\\bb1.wav" );
				lstrcpy( DTMFWaveItems[2].szWavePath, "C:\\Kalimba\\Audio\\bb2.wav" );
				lstrcpy( DTMFWaveItems[3].szWavePath, "C:\\Kalimba\\Audio\\bb3.wav" );
				lstrcpy( DTMFWaveItems[4].szWavePath, "C:\\Kalimba\\Audio\\bb4.wav" );
				lstrcpy( DTMFWaveItems[5].szWavePath, "C:\\Kalimba\\Audio\\bb5.wav" );
				lstrcpy( DTMFWaveItems[6].szWavePath, "C:\\Kalimba\\Audio\\bb6.wav" );
				lstrcpy( DTMFWaveItems[7].szWavePath, "C:\\Kalimba\\Audio\\bb7.wav" );
				lstrcpy( DTMFWaveItems[8].szWavePath, "C:\\Kalimba\\Audio\\bb8.wav" );
				lstrcpy( DTMFWaveItems[9].szWavePath, "C:\\Kalimba\\Audio\\bb9.wav" );
			}


			lstrcpy( DTMFWaveItems[10].szWavePath, "C:\\Kalimba\\Audio\\a.wav" );
			lstrcpy( DTMFWaveItems[11].szWavePath, "C:\\Kalimba\\Audio\\b.wav" );
			lstrcpy( DTMFWaveItems[12].szWavePath, "C:\\Kalimba\\Audio\\c.wav" );
			lstrcpy( DTMFWaveItems[13].szWavePath, "C:\\Kalimba\\Audio\\d.wav" );
			lstrcpy( DTMFWaveItems[14].szWavePath, "C:\\Kalimba\\Audio\\pound.wav" );
			lstrcpy( DTMFWaveItems[15].szWavePath, "C:\\Kalimba\\Audio\\star.wav" );
			lstrcpy( DTMFWaveItems[16].szWavePath, "C:\\Kalimba\\Audio\\2600.wav" );
			lstrcpy( DTMFWaveItems[17].szWavePath, "C:\\Kalimba\\Audio\\5c.wav" );
			lstrcpy( DTMFWaveItems[18].szWavePath, "C:\\Kalimba\\Audio\\10c.wav" );
			lstrcpy( DTMFWaveItems[19].szWavePath, "C:\\Kalimba\\Audio\\25c.wav" );
			lstrcpy( DTMFWaveItems[20].szWavePath, "C:\\Kalimba\\Audio\\Kp.wav" );
			lstrcpy( DTMFWaveItems[21].szWavePath, "C:\\Kalimba\\Audio\\ST.wav" );
			lstrcpy( DTMFWaveItems[22].szWavePath, "C:\\Kalimba\\Audio\\stp.wav" );
			lstrcpy( DTMFWaveItems[23].szWavePath, "C:\\Kalimba\\Audio\\stp2.wav" );
			lstrcpy( DTMFWaveItems[24].szWavePath, "C:\\Kalimba\\Audio\\stp3.wav" );

			//Coin Return corresponds to blue box Kp key

			lstrcpy( DTMFWaveItems[25].szWavePath, "C:\\Kalimba\\Audio\\Kp.wav" );

			//
			// Iterate through all the wave files and store the wave
			// data in the structure array (DTMFWaveItems[]).

			for ( i = 0; i < MAX_DTMF_TONES; i ++ )	{

					memset( &DTMFWaveItems[i].ChunkInfo, 0, sizeof( MMCKINFO ));

					DTMFWaveItems[i].WaveData = NULL;

					hWaveHandle = mmioOpen( DTMFWaveItems[i].szWavePath, 0, MMIO_READ );
					if ( !hWaveHandle )	{
						MessageBox( hWnd, DTMFWaveItems[i].szWavePath, "Wave Open Error", 0 );
					}

					Res = mmioDescend( hWaveHandle, &DTMFWaveItems[i].ChunkInfo,
										0, MMIO_FINDRIFF );

					CheckMMIOError( hWnd, Res );

					DTMFWaveItems[i].FormatChunkInfo.ckid = mmioStringToFOURCC( "fmt", 0 );

					Res = mmioDescend( hWaveHandle, &DTMFWaveItems[i].FormatChunkInfo,
										&DTMFWaveItems[i].ChunkInfo, MMIO_FINDCHUNK );
					
					CheckMMIOError( hWnd, Res );

					memset( &DTMFWaveItems[i].WaveFmt, 0, sizeof( WAVEFORMATEX ));

					mmioRead( hWaveHandle, (char *)&DTMFWaveItems[i].WaveFmt,
								DTMFWaveItems[i].FormatChunkInfo.cksize );

					Res = mmioAscend( hWaveHandle, 
								&DTMFWaveItems[i].FormatChunkInfo, 0 );

					CheckMMIOError( hWnd, Res );

					DTMFWaveItems[i].DataChunkInfo.ckid = mmioStringToFOURCC( "data", 0 );
											
					Res = mmioDescend( hWaveHandle, &DTMFWaveItems[i].DataChunkInfo,
							&DTMFWaveItems[i].ChunkInfo, MMIO_FINDCHUNK );

					DTMFWaveItems[i].DataSize = DTMFWaveItems[i].DataChunkInfo.cksize;

					if ( DTMFWaveItems[i].WaveData != NULL )
						free( DTMFWaveItems[i].WaveData );

					DTMFWaveItems[i].WaveData = 
						(char *)malloc( DTMFWaveItems[i].DataSize * sizeof( char ));
		

					mmioRead( hWaveHandle, DTMFWaveItems[i].WaveData, 
								DTMFWaveItems[i].DataSize );

					mmioClose( hWaveHandle, 0 );

			}

			// Get the number of wave compatible devices installed.
			// 0 is returned if none are present...

			gnNumberOfWaveDevices = waveOutGetNumDevs();

			// What we want is a voice modem capable of sound...

//			wsprintf( szTmp, "Number of devices = %u", gnNumberOfWaveDevices );
//			MessageBox( hWnd, szTmp, "Wave Devices", MB_OK );

			for ( i = 0; i < gnNumberOfWaveDevices; i++ )	{

				memset( &pwoc, 0, sizeof( pwoc ));

				if ( waveOutGetDevCaps( i, &pwoc, sizeof( pwoc )) == MMSYSERR_NOERROR )	{

//					if ( i == 2 )	{
//						wsprintf( szTmp, "%s is DevId:%d", pwoc.szPname, i );
//						lstrcpy( szWaveDevName, pwoc.szPname );
//						MessageBox( hWnd, szTmp, "Device Found", 0 );
//						wsprintf( szTmp, "Channels = %ld", pwoc.wChannels );

				// Useful to know...

//					gnWaveDevToUse = i;
					wNumWaveChannels = pwoc.wChannels;
					
//					MessageBox( hWnd, szTmp, "Channels", 0 );
//					wsprintf( szTmp, "Capability = %x ", pwoc.dwFormats );
//					MessageBox( hWnd, szTmp, "Formats", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_1M08 )
//						MessageBox( hWnd, "Format 1M08", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_1M16 )
//						MessageBox( hWnd, "Format 1M16", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_1S08 )
//						MessageBox( hWnd, "Format 1S08", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_1S16 )
//						MessageBox( hWnd, "Format 1S16", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_2M08 )
//						MessageBox( hWnd, "Format 2M08", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_2M16 )
//						MessageBox( hWnd, "Format 2M16", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_2S08 )
//						MessageBox( hWnd, "Format 2S08", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_2S16 )
//						MessageBox( hWnd, "Format 2S16", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_4M08 )
//						MessageBox( hWnd, "Format 4M08", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_4M16 )
//						MessageBox( hWnd, "Format 4M16", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_4S08 )
//						MessageBox( hWnd, "Format 4S08", "Format", 0 );

//					if ( pwoc.dwFormats && WAVE_FORMAT_4S16 )
//						MessageBox( hWnd, "Format 4S16", "Format", 0 ); 
//					}
				}
			}

			if ( uToneStandard == TONE_DTMF )
				SendDlgItemMessage( hWnd, IDC_TG_DTMF,
						BM_SETCHECK, (WPARAM)TRUE, 0 );

			if ( uToneStandard == TONE_CCITT )
				SendDlgItemMessage( hWnd, IDC_TG_CCITT,
						BM_SETCHECK, (WPARAM)TRUE, 0 );


			SendDlgItemMessage( hWnd, IDC_TG_DEVNAME,
						EM_REPLACESEL, 0, (LPARAM)pwoc.szPname );
			
			gnWaveDevToUse = 2;
			return FALSE;

		case WM_COMMAND:
			
			switch( LOWORD( wParam ))	{

				// 
				// The CCITT radio button was selected.
				// So load up all the associated wave files
				// and store the wave data.
				//

				case IDC_TG_CCITT:

					uToneStandard = TONE_CCITT;
					lstrcpy( DTMFWaveItems[0].szWavePath, "C:\\Kalimba\\Audio\\0ccitt.wav" );
					lstrcpy( DTMFWaveItems[1].szWavePath, "C:\\Kalimba\\Audio\\1ccitt.wav" );
					lstrcpy( DTMFWaveItems[2].szWavePath, "C:\\Kalimba\\Audio\\2ccitt.wav" );
					lstrcpy( DTMFWaveItems[3].szWavePath, "C:\\Kalimba\\Audio\\3ccitt.wav" );
					lstrcpy( DTMFWaveItems[4].szWavePath, "C:\\Kalimba\\Audio\\4ccitt.wav" );
					lstrcpy( DTMFWaveItems[5].szWavePath, "C:\\Kalimba\\Audio\\5ccitt.wav" );
					lstrcpy( DTMFWaveItems[6].szWavePath, "C:\\Kalimba\\Audio\\6ccitt.wav" );
					lstrcpy( DTMFWaveItems[7].szWavePath, "C:\\Kalimba\\Audio\\7ccitt.wav" );
					lstrcpy( DTMFWaveItems[8].szWavePath, "C:\\Kalimba\\Audio\\8ccitt.wav" );
					lstrcpy( DTMFWaveItems[9].szWavePath, "C:\\Kalimba\\Audio\\9ccitt.wav" );

					BuildMMIOToneItems( hWnd );

					return FALSE;

				// 
				// The DTMF radio button was selected.
				// So load up all the associated wave files
				// and store the wave data.
				//

				case IDC_TG_DTMF:

					uToneStandard = TONE_DTMF;
					lstrcpy( DTMFWaveItems[0].szWavePath, "C:\\Kalimba\\Audio\\0.wav" );
					lstrcpy( DTMFWaveItems[1].szWavePath, "C:\\Kalimba\\Audio\\1.wav" );
					lstrcpy( DTMFWaveItems[2].szWavePath, "C:\\Kalimba\\Audio\\2.wav" );
					lstrcpy( DTMFWaveItems[3].szWavePath, "C:\\Kalimba\\Audio\\3.wav" );
					lstrcpy( DTMFWaveItems[4].szWavePath, "C:\\Kalimba\\Audio\\4.wav" );
					lstrcpy( DTMFWaveItems[5].szWavePath, "C:\\Kalimba\\Audio\\5.wav" );
					lstrcpy( DTMFWaveItems[6].szWavePath, "C:\\Kalimba\\Audio\\6.wav" );
					lstrcpy( DTMFWaveItems[7].szWavePath, "C:\\Kalimba\\Audio\\7.wav" );
					lstrcpy( DTMFWaveItems[8].szWavePath, "C:\\Kalimba\\Audio\\8.wav" );
					lstrcpy( DTMFWaveItems[9].szWavePath, "C:\\Kalimba\\Audio\\9.wav" );

					BuildMMIOToneItems( hWnd );

					return FALSE;

				// 
				// The DTMF radio button was selected.
				// So load up all the associated wave files
				// and store the wave data.
				//

				case IDC_TG_BLUEBOX:

					uToneStandard = TONE_BLUEBOX;
					lstrcpy( DTMFWaveItems[0].szWavePath, "C:\\Kalimba\\Audio\\bb10.wav" );
					lstrcpy( DTMFWaveItems[1].szWavePath, "C:\\Kalimba\\Audio\\bb1.wav" );
					lstrcpy( DTMFWaveItems[2].szWavePath, "C:\\Kalimba\\Audio\\bb2.wav" );
					lstrcpy( DTMFWaveItems[3].szWavePath, "C:\\Kalimba\\Audio\\bb3.wav" );
					lstrcpy( DTMFWaveItems[4].szWavePath, "C:\\Kalimba\\Audio\\bb4.wav" );
					lstrcpy( DTMFWaveItems[5].szWavePath, "C:\\Kalimba\\Audio\\bb5.wav" );
					lstrcpy( DTMFWaveItems[6].szWavePath, "C:\\Kalimba\\Audio\\bb6.wav" );
					lstrcpy( DTMFWaveItems[7].szWavePath, "C:\\Kalimba\\Audio\\bb7.wav" );
					lstrcpy( DTMFWaveItems[8].szWavePath, "C:\\Kalimba\\Audio\\bb8.wav" );
					lstrcpy( DTMFWaveItems[9].szWavePath, "C:\\Kalimba\\Audio\\bb9.wav" );

					BuildMMIOToneItems( hWnd );

					return FALSE;

				//
				// The button #1 was pressed
				//
				case IDC_TG_1:

					// Set index to 1

					uWaveItemToPlay = 1;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #2 was pressed
				//

				case IDC_TG_2:

					// Set index to 2

					uWaveItemToPlay = 2;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #3 was pressed
				//

				case IDC_TG_3:

					// Set index to 3

					uWaveItemToPlay = 3;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;


				//
				// The button #4 was pressed
				//

				case IDC_TG_4:

					// Set index to 4

					uWaveItemToPlay = 4;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #5 was pressed
				//

				case IDC_TG_5:

					// Set index to 5

					uWaveItemToPlay = 5;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #6 was pressed
				//

				case IDC_TG_6:

					// Set index to 6

					uWaveItemToPlay = 6;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #7 was pressed
				//

				case IDC_TG_7:

					// Set index to 7

					uWaveItemToPlay = 7;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #8 was pressed
				//

				case IDC_TG_8:

					// Set index to 8

					uWaveItemToPlay = 8;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #9 was pressed
				//

				case IDC_TG_9:

					// Set index to 9

					uWaveItemToPlay = 9;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button #0 was pressed
				//

				case IDC_TG_0:

					// Set index to 0

					uWaveItemToPlay = 0;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'A' was pressed
				//

				case IDC_TG_A:

					// Set index to 10

					uWaveItemToPlay = 10;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'B' was pressed
				//

				case IDC_TG_B:

					// Set index to 11

					uWaveItemToPlay = 11;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'C' was pressed
				//

				case IDC_TG_C:

					// Set index to 12

					uWaveItemToPlay = 12;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'D' was pressed
				//

				case IDC_TG_D:

					// Set index to 13

					uWaveItemToPlay = 13;
					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button '#' was pressed
				//

				case IDC_TG_POUND:

					// Set index to 14

					uWaveItemToPlay = 14;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button '*' was pressed
				//

				case IDC_TG_SPLAT:

					// Set index to 15

					uWaveItemToPlay = 15;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button '2600 Hz' was pressed
				//

				case IDC_TG_T2600:

					// Set index to 16

					uWaveItemToPlay = 16;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button '5 cents' was pressed
				//

				case IDC_TG_NICKEL:

					// Set index to 17

					uWaveItemToPlay = 17;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button '10 cents' was pressed
				//

				case IDC_TG_DIME:

					// Set index to 18

					uWaveItemToPlay = 18;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button '25 cents' was pressed
				//

				case IDC_TG_QUARTER:

					// Set index to 19

					uWaveItemToPlay = 19;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;


				//
				// The button 'KP' was pressed
				//

				case IDC_TG_KEYPULSEF:

					// Set index to 20

					uWaveItemToPlay = 20;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'ST' was pressed
				//

				case IDC_TG_KPFSTOP:

					// Set index to 21

					uWaveItemToPlay = 21;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'ST1p' was pressed
				//

				case IDC_TG_STP1:

					// Set index to 22

					uWaveItemToPlay = 22;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'ST2p' was pressed
				//

				case IDC_TG_ST2P:

					// Set index to 23

					uWaveItemToPlay = 23;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'ST3p' was pressed
				//

				case IDC_TG_ST3P:

					// Set index to 24

					uWaveItemToPlay = 24;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;


				//
				// The button 'coin return' was pressed
				//***FOOBAR*** No wave file for this yet

				case IDC_TG_COIN_RET:

					// Set index to 25

					uWaveItemToPlay = 25;

					if ( fInUse )
						return FALSE;

					fInUse = TRUE;

					PlayWaveData( hWnd, uWaveItemToPlay );

					return FALSE;

				//
				// The button 'CANCEL' was pressed
				//

				case IDC_TG_CANCEL:	

					EndDialog( hWnd, 0 );
					fRet = TRUE;
					return TRUE;

			}
			break;
		
	}
	return FALSE;
}

//
//*************************************************************************
//* BuildMMIOToneItems()
//*
//* A function to reload wave files for digits 0 - 9 when the telecom
//* standard is changed by a user from DTMF to CCITT 5.
//*
//* Parameters:
//*		HWND		- Handle to calling Window/Dialog
//*
//*	Returns:
//*		Nothing.
//*
//*************************************************************************
//

void BuildMMIOToneItems( HWND hWnd )	{

	INT		i;

	for ( i = 0; i < 10; i++ )	{

		memset( &DTMFWaveItems[i].ChunkInfo, 0, sizeof( MMCKINFO ));

		DTMFWaveItems[i].WaveData = NULL;

		hWaveHandle = mmioOpen( DTMFWaveItems[i].szWavePath, NULL, MMIO_READ );
		if ( !hWaveHandle )	{
				MessageBox( hWnd, DTMFWaveItems[i].szWavePath, "Wave Open Error", 0 );
		}

		Res = mmioDescend( hWaveHandle, &DTMFWaveItems[i].ChunkInfo,
								0, MMIO_FINDRIFF );

		CheckMMIOError( hWnd, Res );

		DTMFWaveItems[i].FormatChunkInfo.ckid = mmioStringToFOURCC( "fmt ", 0 );

		Res = mmioDescend( hWaveHandle, &DTMFWaveItems[i].FormatChunkInfo,
							&DTMFWaveItems[i].ChunkInfo, MMIO_FINDCHUNK );
					
		CheckMMIOError( hWnd, Res );

		memset( &DTMFWaveItems[i].WaveFmt, 0, sizeof( WAVEFORMATEX ));

		mmioRead( hWaveHandle, (char *)&DTMFWaveItems[i].WaveFmt,
					DTMFWaveItems[i].FormatChunkInfo.cksize );

		Res = mmioAscend( hWaveHandle, 
					&DTMFWaveItems[i].FormatChunkInfo, 0 );

		CheckMMIOError( hWnd, Res );

		DTMFWaveItems[i].DataChunkInfo.ckid = mmioStringToFOURCC( "data", 0 );
											
		Res = mmioDescend( hWaveHandle, &DTMFWaveItems[i].DataChunkInfo,
						&DTMFWaveItems[i].ChunkInfo, MMIO_FINDCHUNK );

		DTMFWaveItems[i].DataSize = DTMFWaveItems[i].DataChunkInfo.cksize;

		if ( DTMFWaveItems[i].WaveData != NULL )
					free( DTMFWaveItems[i].WaveData );

		DTMFWaveItems[i].WaveData = (char *)malloc( DTMFWaveItems[i].DataSize * sizeof( char ));
		
		mmioRead( hWaveHandle, DTMFWaveItems[i].WaveData, 
						DTMFWaveItems[i].DataSize );

		mmioClose( hWaveHandle, 0 );
	}

}

//
//******************************************************************
//* Function: PlayWaveData()
//*
//* This function is called whenever an item in the wave data array
//* needs to be played.
//*
//* Parameters:
//*		HWND		- Handle to the calling Window/Dialog
//*		UINT		- Index of the wave data item to play
//*
//* Returns:
//*		Nothing.
//******************************************************************
//

void PlayWaveData( HWND hWnd, UINT	uWaveDataItemToPlay )	{

	wsprintf( szTmp, "Sending KeyPad Manual Tone: %s.\r\n", 
			DTMFWaveItems[uWaveDataItemToPlay].szWavePath );
	SendDlgItemMessage( hWndKeyPadDlgOwner, 
			IDC_EDIT_STATUSMSGS, 
			EM_REPLACESEL, 0, (LPARAM)szTmp );


//	Res = waveOutOpen( &hWaveOutDev, 2, 
//						&DTMFWaveItems[uWaveDataItemToPlay].WaveFmt, 
//						0, 0, WAVE_FORMAT_QUERY|WAVE_MAPPED );

	Res = waveOutOpen( &hWaveOutDev, WAVE_MAPPER, 
						&DTMFWaveItems[uWaveDataItemToPlay].WaveFmt, 
						0, 0, WAVE_FORMAT_QUERY );


	CheckWaveError( hWnd, Res, "waveOutOpenCheck" );

	if ( Res )
		return;

//	Res = waveOutOpen( &hWaveOutDev, 2, 
//						&DTMFWaveItems[uWaveDataItemToPlay].WaveFmt,
//						MAKELONG( hWnd, 0 ),
//						0, CALLBACK_WINDOW|WAVE_MAPPED );


	Res = waveOutOpen( &hWaveOutDev, WAVE_MAPPER, 
						&DTMFWaveItems[uWaveDataItemToPlay].WaveFmt,
						MAKELONG( hWnd, 0 ),
						0, CALLBACK_WINDOW );

	CheckWaveError( hWnd, Res, "waveOutOpen" );

	memset( &WaveHeader, 0, sizeof( WaveHeader ));

	WaveHeader.lpData = DTMFWaveItems[uWaveDataItemToPlay].WaveData;
	WaveHeader.dwBufferLength = DTMFWaveItems[uWaveDataItemToPlay].DataSize;

	// Prepare wave header

	Res = waveOutPrepareHeader( hWaveOutDev, &WaveHeader,
					sizeof( WAVEHDR ));

	CheckWaveError( hWnd, Res, "wavePrepare" );

	// Play the sound file, then wait for WM_WOM_DONE

	Res = waveOutWrite( hWaveOutDev, &WaveHeader,
					sizeof( WAVEHDR ));
					
	CheckWaveError( hWnd, Res, "waveOutWrite" );

	return;

}

//
//******************************************************************
//* Function: CheckWaveError()
//*
//* This function is called after any wave play function is used
//* to display any error(s).
//*
//* Parameters:
//*		HWND		- Handle to the calling Window/Dialog.
//*		DWORD		- RetCode from play operation.
//*		char *		- A string to be used with the error code.
//*
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*

void CheckWaveError( HWND hWnd, DWORD Res, char *str )	{

	char buff[256];
	char buff2[256];
	if ( Res == 0 )
		return;
	waveOutGetErrorText( Res, buff, sizeof( buff ));
	wsprintf( buff2, "%s/%s", buff, str );
	MessageBox( hWnd, buff2, "Wave Error", MB_OK );

}

//
//******************************************************************
//* Function: CheckMMIOError()
//*
//* This function is called after any mmio function is used
//* to display any error(s).
//*
//* Parameters:
//*		HWND		- Handle to the calling Window/Dialog.
//*		DWORD		- RetCode from mmio operation.
//*
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*


void CheckMMIOError( HWND hWnd, DWORD Res )	{

	char buff[256];

	if ( Res == 0 )
		return;

	wsprintf( buff, "MMIO Error. Error Code: %d", Res );
	MessageBox( hWnd, buff, "Wave Error", MB_OK );

}

//
//*	End of KeyPad.c
//
