//*
//******************************************************************
//* Module:	PBEdit.c
//*
//*	This modules contains all the functions that will allow
//* the user to create/maintain Phone book files
//*
//* Author:			Byron D Copeland
//*	Date:			September 9, 1999
//*
//******************************************************************
//*

#include <windows.h>
#include <string.h>
#include <commctrl.h>
#include <commdlg.h>
#include <malloc.h>
#include <dlgs.h>

//*
//* Bring in the local header files...
//*	Slurp!
//*

#include "Globals.h"
#include "Resource.h"
#include "Database.h"

BOOL	fItemOneLoaded = FALSE;

//*
//* Prototypes we use in this module.
//*

LPARAM EditShowDialDir( HWND, WORD, HWND, WORD );
BOOL CALLBACK PBEditShowDialogProc( HWND, UINT, WPARAM, LPARAM );
LRESULT PBEditListViewNotify( HWND, LPARAM );
void PBEditAddPhoneNumber( HWND );
BOOL CALLBACK PBEditAddPhoneNumberProc( HWND, UINT, WPARAM, LPARAM );
BOOL InitListView( HWND );

INT GetMaxListViewItems( void );
PLVDATAITEM GetItemData( INT );
PLVDATAITEM GetDatabaseItem( INT );
INT LoadMainCache( INT, INT );
INT SetDatabaseItem( INT, PLVDATAITEM );
// Prototypes of external functions defined elsewhere

void InitDatabase( void );
void LoadTopCache( void );
void LoadBotCache( void );
LPARAM SavePhoneBook( HWND, WORD, HWND, WORD );

//********

LPARAM EditShowDialDir( HWND hWnd, WORD idItem, HWND hwndCtl,
							 WORD wNotifyCode )	{

	DialogBox( hInst, MAKEINTRESOURCE( IDD_PHONEDIRDIALOG ), 
						hWnd, PBEditShowDialogProc );

	return 0;
}


//*
//******************************************************************
//* Function: PBEditShowDialogProc()
//*
//* This Function processes all messages from IDD_PHONEDIRDIALOG
//* DialogBox.
//*
//*	Parameters:
//*		HWND
//*		UINT
//*		WPARAM
//*		LPARAM
//*
//*	Returns:
//*		BOOL
//*
//******************************************************************
//*

BOOL CALLBACK PBEditShowDialogProc( HWND hWnd, UINT wMsg, 
									WPARAM wParam,
									LPARAM lParam )	{

	RECT				rect;
	HIMAGELIST			himlLarge, himlSmall;
	HICON				hIcon;
	INT					i;
	INT					nHeight = 20;
	TCHAR				szTmp[100] = TEXT("");
	DWORD				dwStyle;

	switch ( wMsg )
	{

		case WM_NOTIFY:

			return PBEditListViewNotify( hWnd, lParam );

		case WM_INITDIALOG:

			GetClientRect( hWnd, &rect );

			dwStyle =   WS_TABSTOP | 
						WS_CHILD | 
						WS_BORDER | 
						WS_VISIBLE |
						WS_VSCROLL |
						LVS_AUTOARRANGE |
						LVS_REPORT | 
						LVS_OWNERDATA |
						LVS_SINGLESEL |
						LVS_EDITLABELS;

			ghwndEditLV = CreateWindowEx( WS_EX_CLIENTEDGE, WC_LISTVIEW, 
							TEXT( "Dial Directory" ),
							dwStyle,
							rect.left, rect.top, 
							rect.right, rect.bottom - 100, hWnd,
							(HMENU)IDC_EDITDIALLISTVIEW,
							hInst, NULL );


//			if ( !IsWindow( hwndLV ))	{
//		
//				return 0;
//			}

			InitListView( ghwndEditLV );

			if ( gnTotalPhoneDirRecords == 0 )
				InitDatabase();

			ListView_SetItemCount( ghwndEditLV, GetMaxListViewItems() );
	
			LoadTopCache();
			LoadBotCache();

			i = GetSystemMetrics( SM_CXICON );
			himlLarge = ImageList_Create( i, i, ILC_COLOR, 2, 0 );

			i = GetSystemMetrics( SM_CXSMICON );

			himlSmall = ImageList_Create( i, i, ILC_COLOR, 2, 0 );

			hIcon = LoadIcon( hInst, TEXT( "ID_DocIcon" ));

			i = ImageList_AddIcon( himlLarge, hIcon );

			hIcon = LoadImage( hInst, TEXT( "ID_DocIcon" ), IMAGE_ICON, 16, 16,
							LR_DEFAULTCOLOR );

			ImageList_AddIcon( himlSmall, hIcon );

			ListView_SetImageList( ghwndEditLV, himlLarge, LVSIL_NORMAL );
			ListView_SetImageList( ghwndEditLV, himlSmall, LVSIL_SMALL );

			ListView_SetExtendedListViewStyle( ghwndEditLV, LVS_EX_GRIDLINES |
											LVS_EX_HEADERDRAGDROP |
											LVS_EX_FULLROWSELECT );


			return TRUE;

		case WM_COMMAND:

			switch (LOWORD(wParam))	{

				case IDC_PBD_SAVE:

					SavePhoneBook( hWnd, 0, NULL, 0 );

					return FALSE;

				case IDC_PBD_DELETE:

					return FALSE;

				case IDC_PBD_CANCEL:

					EndDialog( hWnd, 0 );
					return TRUE;
			}
		break;
	}
    return FALSE;

}


//*
//******************************************************************
//* Function:	PBEditListViewNotify()
//*
//* This function is called when a WM_NOTIFY message is processed
//* by the Edit ListView Dialog Box.
//*
//*	Parameters:
//*		HWND			- Handle to call window/dialog box
//*		LPARAM			- LONG Parameter
//*
//*	Returns:
//*		LRESULT
//*
//******************************************************************
//*

LRESULT PBEditListViewNotify( HWND hWnd, LPARAM lParam )	{

	LPNMHDR  pnmh = (LPNMHDR) lParam;
	HWND     hwndListView = GetDlgItem(hWnd, IDC_EDITDIALLISTVIEW);

	LPNMLISTVIEW		pnmlv;
	NMLVDISPINFO		*pLVdi;
	PLVDATAITEM			pdi;
	LPNMLVCACHEHINT		pLVch;
	PLVDATAITEM			pLVdbitem;
	HWND				hwndEdit;

//	hwndEdit = pnmh->hwndFrom;
	pnmlv = (LPNMLISTVIEW)lParam;

	switch( pnmh->code )	{

//		case LVN_HOTTRACK:
//			ListView_SetHotCursor( hwndEdit, LoadCursor( NULL, IDC_NO ));
//			return 0;

		case LVN_COLUMNCLICK:
//			MessageBox( hWnd, "Got Column Click", "PB Edit", 0 );
			return 0;

		// 
		// Process the begin edit label message.  This message is generated
		// when a user clicks twice in a item 0 column
		//
		case LVN_BEGINLABELEDIT:

			pLVdi = (NMLVDISPINFO *)lParam;
			pdi = GetItemData( pLVdi->item.iItem );

			pLVdi->item.cchTextMax = 5;
			switch( pLVdi->item.iSubItem )	{

				case 2:
				case 3:
					return 0;
			}
//			MessageBox( hWnd, "Got LVN_BEGINLABELEDIT", "PB Edit", 0 );

			hwndEdit = ListView_GetEditControl( ghwndEditLV );

			// Make sure we cut text off at 19 chars
			SendDlgItemMessage( hwndEdit, IDC_EDITDIALLISTVIEW, EM_LIMITTEXT, 
							(WPARAM)19, 0 );

			return 0;

		// 
		// Process the end edit label message.  This message is generated
		// when a user presses return after editing an item 
		//

		case LVN_ENDLABELEDIT:

			pLVdi = (NMLVDISPINFO *)lParam;
			pdi = GetItemData( pLVdi->item.iItem );
			pLVdi->item.cchTextMax = 5;

			if ( pLVdi->item.pszText != NULL )	{

//				if ( lstrlen( pLVdi->item.pszText ) > 20 )
//					pLVdi->item.pszText[19] = '\0';
//				wsprintf( szTmp, "%s", pLVdi->item.pszText );

				lstrcpy( pdi->Number, pLVdi->item.pszText );

				//MessageBox( hWnd, "DIALOG STARTING", "PB Edit", 0 );
				DialogBox( hInst, MAKEINTRESOURCE( IDD_PBD_DESCRIPTION ), 
						hWnd, PBEditAddPhoneNumberProc );

				lstrcpy( pdi->Title, szTextOut );

				SetDatabaseItem( pLVdi->item.iItem, pdi );

				fItemOneLoaded = TRUE;
				return TRUE;
			}

			return 0;

		case LVN_ITEMCHANGED:

			pLVdi = (NMLVDISPINFO *)lParam;

			pdi = GetItemData( pLVdi->item.iItem );

			pLVdbitem = GetDatabaseItem( pnmlv->iItem );

			return TRUE;

		case LVN_SETDISPINFO:

			MessageBox( hWnd, "Got LVN_SETDISPINFO", "PB Edit", 0 );

			pLVdi = (NMLVDISPINFO *)lParam;

			pdi = GetItemData( pLVdi->item.iItem );

			if ( pLVdi->item.mask & LVIF_IMAGE )
				pLVdi->item.iImage = pdi->nImage;

			if ( pLVdi->item.mask & LVIF_PARAM )
				pLVdi->item.lParam = 0;

			if ( pLVdi->item.mask & LVIF_STATE )
				pLVdi->item.state = pdi->nState;

			if ( pLVdi->item.mask & LVIF_TEXT )	{

				switch( pLVdi->item.iSubItem )	{

					case 0:
						lstrcpy( pdi->Number, pLVdi->item.pszText );
						break;
					case 1:
						lstrcpy( pdi->Title, pLVdi->item.pszText );
						break;
					case 2:
						lstrcpy( pdi->LastCalled, pLVdi->item.pszText );
						break;

					case 3:
						lstrcpy( pdi->LastStatus, pLVdi->item.pszText );
						break;
				}
				SetDatabaseItem( pLVdi->item.iItem, pdi );
			}

			break;

		//
		// Prccess the get display info message.  This message will occur
		// whenever the listview needs to be updated.  The data is pull
		// from the database then place in the listview.
		//
		case LVN_GETDISPINFO:

			pLVdi = (NMLVDISPINFO *)lParam;

			// Grab a database display item 

			pdi = GetItemData( pLVdi->item.iItem );

			if ( pLVdi->item.mask & LVIF_IMAGE )
				pLVdi->item.iImage = pdi->nImage;

			if ( pLVdi->item.mask & LVIF_PARAM )
				pLVdi->item.lParam = 0;

			if ( pLVdi->item.mask & LVIF_STATE )
				pLVdi->item.state = pdi->nState;

			if ( pLVdi->item.mask & LVIF_TEXT )	{

				if ( fItemOneLoaded )	{
					LoadTopCache();
					LoadBotCache();
				}

				switch( pLVdi->item.iSubItem )	{

					case 0:

						lstrcpy( pLVdi->item.pszText, pdi->Number );
						break;

					case 1:

						lstrcpy( pLVdi->item.pszText, pdi->Title );
						break;

					case 2:
						
							lstrcpy( pLVdi->item.pszText, pdi->LastCalled );

						break;

					case 3:

							lstrcpy( pLVdi->item.pszText, pdi->LastStatus );

						break;
				}
			}
			break;

		case LVN_ODCACHEHINT:

			pLVch = (LPNMLVCACHEHINT)lParam;
			LoadMainCache( pLVch->iFrom, pLVch->iTo );
			break;

		case LVN_ODFINDITEM:

			return TRUE;

	}
	return 0;
}

//*
//******************************************************************
//* Function:	PBEditAddPhoneNumber()
//*
//*	This function displays a dialog box that will allow
//*	the user to add/change a phonebook entry
//*
//*	Parameter:
//*		HWND			- Handle to calling window/dialog box
//*
//*	Returns:
//*		Nothing.
//*
//******************************************************************
//*

void PBEditAddPhoneNumber( HWND hWnd )	{

	DialogBox( hInst, MAKEINTRESOURCE( IDD_PBD_DESCRIPTION ), 
						hWnd, PBEditAddPhoneNumberProc );

	return;
}

//*
//******************************************************************
//* Function:	PBEditAddPhoneNumberProc()
//*
//*	Processing loop for PBEditAddPhoneNumber.
//*
//* Parameters:
//*		HWND
//*		UINT
//*		WPARAM
//*		LPARAM
//*
//*	Returns:
//*		BOOL
//*
//******************************************************************
//*

BOOL CALLBACK PBEditAddPhoneNumberProc( HWND hWnd, UINT wMsg, WPARAM wParam,
					 LPARAM lParam )	{

	INT			i;
	TCHAR		szTmp[50] = TEXT("");

	switch( wMsg )	{

		case WM_INITDIALOG:

			SendDlgItemMessage( hWnd, IDC_PBD_EDITDESCRIPTION, EM_LIMITTEXT, 
							(WPARAM)39, 0 );
			SetFocus( GetDlgItem( hWnd, IDC_PBD_EDITDESCRIPTION ));
			return FALSE;

		case WM_COMMAND:

			switch(LOWORD(wParam))	{

				case IDOK:

					// Add the new description to the list.

					i = SendDlgItemMessage( hWnd, IDC_PBD_EDITDESCRIPTION, 
								EM_LINELENGTH, (WPARAM)0, 0 );
					i++;
					SendDlgItemMessage( hWnd, IDC_PBD_EDITDESCRIPTION, 
										WM_GETTEXT, (WPARAM)i,
										(LPARAM)(LPCSTR)szTmp );

					lstrcpy( szTextOut, szTmp );
//
//					lstrcpy( szTmp, szTmp2 );
//					lstrcat( szTmp, "-" );

//					i = SendDlgItemMessage( hWnd, IDC_PD_ADD_PREFIX, 
//								EM_LINELENGTH, (WPARAM)0, 0 );
//					i++;
//					SendDlgItemMessage( hWnd, IDC_PD_ADD_PREFIX, WM_GETTEXT, (WPARAM)i,
//								(LPARAM)(LPCSTR)szTmp2 );

//					lstrcat( szTmp, szTmp2 );
//					lstrcat( szTmp, "-" );

//					i = SendDlgItemMessage( hWnd, IDC_PD_ADD_SUFFIX, 
//								EM_LINELENGTH, (WPARAM)0, 0 );
//					i++;
//					SendDlgItemMessage( hWnd, IDC_PD_ADD_SUFFIX, WM_GETTEXT, (WPARAM)i,
//								(LPARAM)(LPCSTR)szTmp2 );

//					lstrcat( szTmp, szTmp2 );

//					i = 0;
//					InitDatabaseItem( i, szTmp, "Edited Phone Number", 
//						 "N/A", "N/A", i + 1000 );

//					AddItem( ghwndEditLV, 0, szTmp, "Edited",
//						"NA", "NA", i + 1000 );

					EndDialog( hWnd, 0 );
					return TRUE;

				case IDCANCEL:

					lstrcpy( szTextOut, "\0" );
					EndDialog( hWnd, 0 );
					return TRUE;
			}
		break;
	}

	return FALSE;
}

//*
//******************************************************************
//* Function:	InitListView()
//*
//* This function initially sets up the columns in the ListView.
//*
//*	Parameters:
//*		HWND			- Handle to the calling window/dialog box
//*
//*	Returns:
//*		BOOL
//*
//******************************************************************
//*

BOOL InitListView(HWND hwndListView)	{

	LV_COLUMN   lvColumn;
	int         i;
	TCHAR       szString[4][20] = {"Phone Number", 
								   "Description", 
								   "Last Called", 
								   "Last Status"};

	//empty the list

	ListView_DeleteAllItems( hwndListView );

	//initialize the columns
	lvColumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvColumn.fmt = LVCFMT_LEFT;
	lvColumn.cx = 120;

	for(i = 0; i < 4; i++)	{

		if ( i == 1 )
			lvColumn.cx = 200;
		else
			lvColumn.cx = 120;

		lvColumn.pszText = szString[i];
		ListView_InsertColumn(hwndListView, i, &lvColumn);
   }
	return TRUE;
}

//*
//*	End of PBEdit.c
//*
