//*
//******************************************************************
//* Module:		Reports.c
//*
//*	This program is used to provide generic event/results logging
//*	functionality.  
//*
//* Written by:			Byron D Copeland
//* Date:				31 Aug 99
//*
//******************************************************************
//*

#include <windows.h>
#include <commctrl.h>


// Global Variables used in this program...

TCHAR	szEventsLogPath[MAX_PATH];
TCHAR	szResultsLogPath[MAX_PATH];

//Prototypes used in this file.

INT LogResults( HWND, LPTSTR );
INT CreateFreshResultsLog( HWND, LPTSTR );
INT LogEvents( HWND, LPTSTR );
INT CreateFreshEventsLog( HWND, LPTSTR );

//*
//**************************************************************
//* Function:	CreateFreshResultsLog()
//*
//* Function to write results strings to a log file...
//*
//* Parameters:
//*		HWND			- Handle to calling window/dialog
//*		LPTSTR			- Pointer to path to create
//*
//*	Returns:
//*		INT				- RetCode
//*
//**************************************************************
//

INT CreateFreshResultsLog( HWND hWnd, LPTSTR pszLogFile )	{
	
	HANDLE		hLog;

	lstrcpy( szResultsLogPath, pszLogFile );

	hLog = CreateFile( szResultsLogPath, GENERIC_WRITE, 
			FILE_SHARE_WRITE | FILE_SHARE_READ, 
			NULL, 
			CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );

	if ( hLog == INVALID_HANDLE_VALUE )	{

		return GetLastError();
	}
	CloseHandle( hLog );
	return 0;
}

//*
//**************************************************************
//* Function:	CreateFreshEventsLog()
//*
//* Function to write events strings to a log file...
//*
//* Parameters:
//*		HWND			- Handle to calling window/dialog
//*		LPTSTR			- Pointer to path to create
//*
//*	Returns:
//*		INT				- RetCode
//*
//**************************************************************
//

INT CreateFreshEventsLog( HWND hWnd, LPTSTR pszLogFile )	{
	
	HANDLE		hLog;

	lstrcpy( szEventsLogPath, pszLogFile );

	hLog = CreateFile( szEventsLogPath, GENERIC_WRITE, 
			FILE_SHARE_WRITE | FILE_SHARE_READ, 
			NULL, 
			CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );

	if ( hLog == INVALID_HANDLE_VALUE )	{

		return GetLastError();
	}
	CloseHandle( hLog );
	return 0;
}

//*
//**************************************************************
//* Function:	LogResults()
//*
//* Function to write results strings to a log file...
//*
//* Parameters:
//*		HWND			- Handle to calling window/dialog
//*		LPTSTR			- Pointer to path to use
//*
//*	Returns:
//*		INT				- RetCode
//*
//**************************************************************
//*

INT LogResults( HWND hWnd, LPTSTR pszLogString )	{

	HANDLE		hLog;
	DWORD		dwBytesWritten;

	hLog = CreateFile( szResultsLogPath, GENERIC_WRITE, 
				FILE_SHARE_WRITE | FILE_SHARE_READ, 
				NULL, 
				OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );

	if ( hLog != INVALID_HANDLE_VALUE )	{

		SetFilePointer( hLog, 0, NULL, FILE_END );

		if ( !WriteFile( hLog, pszLogString, 
			lstrlen( pszLogString ), &dwBytesWritten, NULL ))	{

			CloseHandle( hLog );
			return GetLastError();
		}
		CloseHandle( hLog );
	}
	else	{
		return GetLastError();
	}
	return 0;
}

//*
//**************************************************************
//* Function:	LogEvents()
//*
//* Function to write events strings to a log file...
//*
//* Parameters:
//*		HWND			- Handle to calling window/dialog
//*		LPTSTR			- Pointer to path to use
//*
//*	Returns:
//*		INT				- RetCode
//*
//**************************************************************
//*

INT LogEvents( HWND hWnd, LPTSTR pszLogString )	{

	HANDLE		hLog;
	DWORD		dwBytesWritten;

	hLog = CreateFile( szEventsLogPath, GENERIC_WRITE, 
				FILE_SHARE_WRITE | FILE_SHARE_READ, 
				NULL, 
				OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );

	if ( hLog != INVALID_HANDLE_VALUE )	{

		SetFilePointer( hLog, 0, NULL, FILE_END );

		if ( !WriteFile( hLog, pszLogString, 
			lstrlen( pszLogString ), &dwBytesWritten, NULL ))	{

			CloseHandle( hLog );
			return GetLastError();
		}
		CloseHandle( hLog );
	}
	else	{
		return GetLastError();
	}
	return 0;
}
