//*
//******************************************************************
//*Module:		Setup.c
//*
//*		This modules contains functions that allow the display of
//* a properties tabsheet type dialog.  Thus allowing the user to
//*	setup communications parameters.
//*
//*Author:		Byron D Copeland
//*Date:		30 Aug 99
//*
//******************************************************************
//*
#include <windows.h>
#include <string.h>
#include <commctrl.h>
#include <commdlg.h>
#include <malloc.h>
#include <dlgs.h>
#include <prsht.h>
#include "Globals.h"
#include "Resource.h"
#include "Properties.h"
#include "UserPref.h"

// Protypes used in this module...

void SetInitialCommunicationsDefaults( void );
void SetCommsDefaultBasicProperties( HWND );
void SetCommsDefaultInitsProperties( HWND );
void SetCommsDefaultMessageProperties( HWND );
void SetCommsDefaultAdvancedProperties( HWND );

LPARAM SerialCommPortSetup( HWND, WORD, HWND, WORD );
BOOL CALLBACK CommSetupBasicDialogProc( HWND, UINT, WPARAM, LPARAM );
BOOL CALLBACK CommSetupInitsDialogProc( HWND, UINT, WPARAM, LPARAM );
BOOL CALLBACK CommSetupMessagesDialogProc( HWND, UINT, WPARAM, LPARAM );
BOOL CALLBACK CommSetupAdvDialogProc( HWND, UINT, WPARAM, LPARAM );


//*
//******************************************************************
//* Function: SetInitialCommunicationsDefaults()
//*
//* Set all configuration parameters on start up of the property
//* sheet.
//*
//* Parameters:
//*		None.
//*
//* Returns:
//*		Nothing.
//*
//******************************************************************
//*

void SetInitialCommunicationsDefaults()	{

	// Basic
	userSavedSettings.dcb.DCBlength = sizeof( userSavedSettings.dcb );

	userSavedSettings.dcb.BaudRate = CBR_19200;
	userSavedSettings.dcb.Parity = NOPARITY;
	userSavedSettings.dcb.ByteSize = 8;
	userSavedSettings.dcb.StopBits = ONESTOPBIT;
	userSavedSettings.dcb.fParity = FALSE;
	userSavedSettings.dcb.fNull = FALSE;
	userSavedSettings.dcb.fBinary = TRUE;
	userSavedSettings.dcb.fOutxCtsFlow = FALSE;
	userSavedSettings.dcb.fOutxDsrFlow = FALSE;
	userSavedSettings.dcb.fDsrSensitivity = TRUE;
	userSavedSettings.dcb.fDtrControl = DTR_CONTROL_ENABLE;
	userSavedSettings.dcb.fOutX = FALSE;
	userSavedSettings.dcb.fInX = FALSE;
	userSavedSettings.dcb.fErrorChar = FALSE;
	userSavedSettings.dcb.fRtsControl = RTS_CONTROL_ENABLE;
	userSavedSettings.dcb.fAbortOnError = FALSE;
	
	lstrcpy( userSavedSettings.szPortInUse, TEXT("Serial Port COM1:" ));
	userSavedSettings.fWantsCr = TRUE;
	// Inits
	lstrcpy( userSavedSettings.szEscapeCode, szDefEscapeCode );
	lstrcpy( userSavedSettings.szOnHook, szDefOnHook );
	lstrcpy( userSavedSettings.szOffHook, szDefOffHook );
	lstrcpy( userSavedSettings.szReturnOnline, szDefReturnOnline );
	lstrcpy( userSavedSettings.szAttention, szDefAttention );
	lstrcpy( userSavedSettings.szDialPrefix, "" );
	lstrcpy( userSavedSettings.szReset, szDefReset );
	lstrcpy( userSavedSettings.szInit1, szDefInit1 );
	lstrcpy( userSavedSettings.szInit2, szDefInit2 );
	// Messages
	lstrcpy( userSavedSettings.szMsgOk, szDefMsgOk );
	lstrcpy( userSavedSettings.szMsgBusy, szDefMsgBusy );
	lstrcpy( userSavedSettings.szMsgRing, szDefMsgRing );
	lstrcpy( userSavedSettings.szMsgVoice, szDefMsgVoice );
	lstrcpy( userSavedSettings.szMsgError, szDefMsgError );
	lstrcpy( userSavedSettings.szMsgNoCarrier, szDefMsgNoCarrier );
	lstrcpy( userSavedSettings.szMsgNoDialTone, szDefMsgNoDialTone );
	lstrcpy( userSavedSettings.szMsgNoAnswer, szDefMsgNoAnswer );
	lstrcpy( userSavedSettings.szMsgConnect1200, szDefMsgCon1200 );
	lstrcpy( userSavedSettings.szMsgConnect2400, szDefMsgCon2400 );
	lstrcpy( userSavedSettings.szMsgConnect4800, szDefMsgCon4800 );
	lstrcpy( userSavedSettings.szMsgConnect9600, szDefMsgCon9600 );
	lstrcpy( userSavedSettings.szMsgConnect14400, szDefMsgCon14400 );
	lstrcpy( userSavedSettings.szMsgConnect19200, szDefMsgCon19200 );
	lstrcpy( userSavedSettings.szMsgConnect38400, szDefMsgCon38400 );
	lstrcpy( userSavedSettings.szMsgConnect56000, szDefMsgCon56000 );
	lstrcpy( userSavedSettings.szMsgConnect57600, szDefMsgCon57600 );
	lstrcpy( userSavedSettings.szMsgConnect115200, szDefMsgCon115200 );
	lstrcpy( userSavedSettings.szMsgConnect128000, szDefMsgCon128000 );
	lstrcpy( userSavedSettings.szMsgConnect256000, szDefMsgCon256000 );
	// Advanced
	lstrcpy( userSavedSettings.szPortAddress, szDefAddressCom1 );
	lstrcpy( userSavedSettings.szPortIRQ, szDefIRQ4 );
	lstrcpy( userSavedSettings.szPortInterrupt, szDefInterrupt2 );
	userSavedSettings.fLockBaud = TRUE;
	userSavedSettings.fLowerDTR = TRUE;
	userSavedSettings.fRTSControl = TRUE;
//	userSavedSettings.dcb.fRtsControl = RTS_CONTROL_ENABLE;
//	userSavedSettings.dcb.fDtrControl = DTR_CONTROL_ENABLE;
	userSavedSettings.fCTSControl = FALSE;
	userSavedSettings.fDSRControl = FALSE;
//	userSavedSettings.fTTYDisplay = FALSE;
	userSavedSettings.fDTRControl = DTR_CONTROL_ENABLE;
	userSavedSettings.dwInQueueSize = 4096;
	userSavedSettings.dwOutQueueSize = 0;
}

//
//******************************************************************
// SetCommsDefaultBasicProperties
//
// Set the initial state of the Communications Properties for 
// Basic things.
//
//******************************************************************
//
void SetCommsDefaultBasicProperties( HWND hWnd )	{

	TCHAR	szDev[55] = TEXT("");
	TCHAR	tmp[20] = TEXT( "" );
	INT		i;


	CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B19200 );

	CheckRadioButton( hWnd, IDC_PARITY_NONE, IDC_PARITY_SPACE,
						IDC_PARITY_NONE );

	CheckRadioButton( hWnd, IDC_WORD8, IDC_WORD4, IDC_WORD8 );

	CheckRadioButton( hWnd, IDC_STOPBITS1, IDC_STOPBITS2, IDC_STOPBITS1 );

	lstrcpy( szDev, TEXT("Serial Port COM1:" ));
	SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDev );

	lstrcpy( szDev, TEXT("Serial Port COM2:" ));
	SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDev );

	lstrcpy( szDev, TEXT("Serial Port COM3:" ));
	SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDev );

	lstrcpy( szDev, TEXT("Serial Port COM4:" ));
	SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDev );

	lstrcpy( szDev, TEXT( "IrComm Port   " ));
	SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDev );

	lstrcpy( szDev, TEXT( "Raw IR Port   " ));
	SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDev );

	// Set Com port to szPortInUse variable

	lstrcpy( szDev, userSavedSettings.szPortInUse );
	i = SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_FINDSTRINGEXACT,
		(WPARAM)-1, (LPARAM)(LPCTSTR)szDev );

	if ( i != CB_ERR )	{
			SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT,
					CB_SETCURSEL, (WPARAM)i, 0 );
	}

	switch( userSavedSettings.dcb.BaudRate )	{

		case CBR_1200:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B1200 );
			break;
		case CBR_2400:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B2400 );
			break;
		case CBR_4800:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B4800 );
			break;
		case CBR_9600:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B9600 );
			break;
		case CBR_14400:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B14400 );
			break;
		case CBR_19200:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B19200 );
			break;
		case CBR_38400:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B38400 );
			break;
		case CBR_56000:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B56000 );
			break;
		case CBR_57600:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B57600 );
			break;
		case CBR_115200:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B115200 );
			break;
		case CBR_128000:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B128000 );
			break;
		case CBR_256000:
			CheckRadioButton( hWnd, IDC_B256000, IDC_B1200, IDC_B256000 );
			break;
	}

//	GetIrCommDeviceName( &szDev[lstrlen ( szDev )]);
//	SendDlgItemMessage( GetDlgItem( hWnd, IDC_CMDBAR ),
//						IDC_COMPORT, CB_INSERTSTRING,
//						-1, (LPARAM)szDev );


//	fCommBasicPropertiesChanged = FALSE;
}

//
//******************************************************************
// SetCommsDefaultInitsProperties
//
// Set the initial state of the Communications Properties for 
// Initialization Strings.
//
//******************************************************************
//
void SetCommsDefaultInitsProperties( HWND hWnd )	{

	TCHAR	szDev[55] = TEXT( "" );

	// Send EM_LIMITTEXT message for all the edit fields in
	// the property sheet.

	SendDlgItemMessage( hWnd, IDC_INIT_ESCAPE_CODE, EM_LIMITTEXT, 
							(WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_INIT_ESCAPE_CODE, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szEscapeCode );


	SendDlgItemMessage( hWnd, IDC_INIT_ONHOOK, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_INIT_ONHOOK, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szOnHook );

	SendDlgItemMessage( hWnd, IDC_INIT_OFFHOOK, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_INIT_OFFHOOK, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szOffHook );


	SendDlgItemMessage( hWnd, IDC_INIT_RETURNONLINE, EM_LIMITTEXT, 
								(WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_INIT_RETURNONLINE, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szReturnOnline );

	SendDlgItemMessage( hWnd, IDC_INIT_ATTENTION, EM_LIMITTEXT, (WPARAM)50,	0 );
	SendDlgItemMessage( hWnd, IDC_INIT_ATTENTION, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szAttention );

	SendDlgItemMessage( hWnd, IDC_INIT_DIALPREFIX, EM_LIMITTEXT, (WPARAM)50,	0 );
	SendDlgItemMessage( hWnd, IDC_INIT_DIALPREFIX, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szDialPrefix );

	SendDlgItemMessage( hWnd, IDC_INIT_RESET, EM_LIMITTEXT, (WPARAM)50,	0 );
	SendDlgItemMessage( hWnd, IDC_INIT_RESET, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szReset );

	SendDlgItemMessage( hWnd, IDC_INIT_INIT1, EM_LIMITTEXT, (WPARAM)50,	0 );
	SendDlgItemMessage( hWnd, IDC_INIT_INIT1, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szInit1 );

	SendDlgItemMessage( hWnd, IDC_INIT_INIT2, EM_LIMITTEXT, (WPARAM)50,	0 );
	SendDlgItemMessage( hWnd, IDC_INIT_INIT2, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szInit2 );

	// set the radio button correctly for "wants CR" on the 
	// property sheet

	if ( userSavedSettings.fWantsCr  )

		SendDlgItemMessage( hWnd, IDC_INIT_CR_YES, BM_SETCHECK,
				(WPARAM)BST_CHECKED, 0 );
	else
		SendDlgItemMessage( hWnd, IDC_INIT_CR_YES, BM_SETCHECK,
				(WPARAM)BST_UNCHECKED, 0 );

//	fCommInitsPropertiesChanged = FALSE;
}


//
//******************************************************************
// SetCommsDefaultMessageProperties
//
// Set the initial state of the Communications Properties for 
// Modem Messages.
//
//******************************************************************
//
void SetCommsDefaultMessageProperties( HWND hWnd )	{

	TCHAR	szDev[55] = TEXT( "" );

	// Send EM_LIMITTEXT message for all the edit fields in
	// the property sheet.  Then stuff the field with the user
	// saved settings...

	SendDlgItemMessage( hWnd, IDC_MSG_OK, EM_LIMITTEXT, (WPARAM)50,	0 );
	SendDlgItemMessage( hWnd, IDC_MSG_OK, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgOk );

	SendDlgItemMessage( hWnd, IDC_MSG_BUSY, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_BUSY, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgBusy );

	SendDlgItemMessage( hWnd, IDC_MSG_RING, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_RING, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgRing );

	SendDlgItemMessage( hWnd, IDC_MSG_VOICE, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_VOICE, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgVoice );

	SendDlgItemMessage( hWnd, IDC_MSG_ERROR, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_ERROR, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgError );

	SendDlgItemMessage( hWnd, IDC_MSG_CARRIER, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_CARRIER, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgNoCarrier );

	SendDlgItemMessage( hWnd, IDC_MSG_DIALTONE, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_DIALTONE, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgNoDialTone );

	SendDlgItemMessage( hWnd, IDC_MSG_ANSWER, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_ANSWER, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgNoAnswer );

	SendDlgItemMessage( hWnd, IDC_MSG_B1200, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B1200, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect1200 );

	SendDlgItemMessage( hWnd, IDC_MSG_B2400, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B2400, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect2400 );

	SendDlgItemMessage( hWnd, IDC_MSG_B4800, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B4800, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect4800 );

	SendDlgItemMessage( hWnd, IDC_MSG_B9600, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B9600, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect9600 );

	SendDlgItemMessage( hWnd, IDC_MSG_B14400, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B14400, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect14400 );

	SendDlgItemMessage( hWnd, IDC_MSG_B19200, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B19200, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect19200 );

	SendDlgItemMessage( hWnd, IDC_MSG_B38400, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B38400, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect38400 );

	SendDlgItemMessage( hWnd, IDC_MSG_B56000, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B56000, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect56000 );

	SendDlgItemMessage( hWnd, IDC_MSG_B57600, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B57600, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect57600 );

	SendDlgItemMessage( hWnd, IDC_MSG_B115200, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B115200, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect115200 );

	SendDlgItemMessage( hWnd, IDC_MSG_B128000, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B128000, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect128000 );

	SendDlgItemMessage( hWnd, IDC_MSG_B256000, EM_LIMITTEXT, (WPARAM)50, 0 );
	SendDlgItemMessage( hWnd, IDC_MSG_B256000, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)userSavedSettings.szMsgConnect256000 );

//	fCommMessagesPropertiesChanged = FALSE;
}

//
//******************************************************************
// SetCommsDefaultAdvancedProperties
//
// Set the initial state of the Communications Properties for 
// Modem Advanced Settings.
//
//******************************************************************
//
void SetCommsDefaultAdvancedProperties( HWND hWnd )	{

	TCHAR	szTmp[20] = TEXT( "" );
	INT		i;

	// Setup the combo box that will have a list of com ports
	// to choose from...

	SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefAddressCom1 );
	SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefAddressCom2 );
	SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefAddressCom3 );
	SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefAddressCom4 );

	i = SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS, CB_FINDSTRINGEXACT,
		(WPARAM)-1, (LPARAM)(LPCTSTR)szDefAddressCom1 );

	if ( i != CB_ERR )	{
			SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS,
					CB_SETCURSEL, (WPARAM)i, 0 );
	}

	// Setup combo box that holds a list of IRQ

	// ***FOOBAR***  This will eventually be
	// removed....

	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ3 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ4 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ5 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ6 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ7 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ8 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ9 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ10 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ11 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ12 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ13 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ14 );
	SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefIRQ15 );


	i = SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_FINDSTRINGEXACT,
		(WPARAM)-1, (LPARAM)(LPCTSTR)szDefIRQ4 );

	if ( i != CB_ERR )	{
			SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE,
					CB_SETCURSEL, (WPARAM)i, 0 );
	}

	SendDlgItemMessage( hWnd, IDC_COMBO_INTERRUPT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefInterrupt1 );
	SendDlgItemMessage( hWnd, IDC_COMBO_INTERRUPT, CB_ADDSTRING, 0,
						(LPARAM)(LPCTSTR)szDefInterrupt2 );

	i = SendDlgItemMessage( hWnd, IDC_COMBO_INTERRUPT, CB_FINDSTRINGEXACT,
		(WPARAM)-1, (LPARAM)(LPCTSTR)szDefInterrupt2 );

	if ( i != CB_ERR )	{
			SendDlgItemMessage( hWnd, IDC_COMBO_INTERRUPT,
					CB_SETCURSEL, (WPARAM)i, 0 );
	}

	if ( userSavedSettings.fRTSControl )
			CheckRadioButton( hWnd, IDC_ADV_HSRTS, IDC_ADV_HSCTS, IDC_ADV_HSRTS );
	if ( userSavedSettings.fDTRControl )
			CheckRadioButton( hWnd, IDC_ADV_HSRTS, IDC_ADV_HSCTS, IDC_ADV_HSDTR );
	if ( userSavedSettings.fDSRControl )
			CheckRadioButton( hWnd, IDC_ADV_HSRTS, IDC_ADV_HSCTS, IDC_ADV_HSDSR );
	if ( userSavedSettings.fCTSControl )
			CheckRadioButton( hWnd, IDC_ADV_HSRTS, IDC_ADV_HSCTS, IDC_ADV_HSCTS );

	if ( userSavedSettings.fLowerDTR  )

		SendDlgItemMessage( hWnd, IDC_ADV_LOWER_DTR, BM_SETCHECK,
				(WPARAM)BST_CHECKED, 0 );
	else
		SendDlgItemMessage( hWnd, IDC_ADV_LOWER_DTR, BM_SETCHECK,
				(WPARAM)BST_UNCHECKED, 0 );

	if ( userSavedSettings.fLockBaud  )

		SendDlgItemMessage( hWnd, IDC_ADV_LOCK_BAUD, BM_SETCHECK,
				(WPARAM)BST_CHECKED, 0 );
	else
		SendDlgItemMessage( hWnd, IDC_ADV_LOCK_BAUD, BM_SETCHECK,
				(WPARAM)BST_UNCHECKED, 0 );

	ultoa( userSavedSettings.dwInQueueSize, szTmp, 10 );
	SendDlgItemMessage( hWnd, IDC_ADV_RCV_QUEUE, EM_LIMITTEXT, (WPARAM)5, 0 );
	SendDlgItemMessage( hWnd, IDC_ADV_RCV_QUEUE, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)szTmp );

	ultoa( userSavedSettings.dwOutQueueSize, szTmp, 10 );
	SendDlgItemMessage( hWnd, IDC_ADV_XMIT_QUEUE, EM_LIMITTEXT, (WPARAM)5, 0 );
	SendDlgItemMessage( hWnd, IDC_ADV_XMIT_QUEUE, EM_REPLACESEL, 0,
						(LPARAM)(LPCTSTR)szTmp );

//	fCommAdvPropertiesChanged = FALSE;
}


//
//******************************************************************
//* SerialCommPortSetup
//*
//* Setup/Process the comm port setup properties.
//* This allows a user to specifically tune the com port settings...
//******************************************************************
//
LPARAM SerialCommPortSetup( HWND hWnd, WORD idItem, HWND hwndCtl,
						   WORD wNotifyCode )	{

	PROPSHEETPAGE	psp[4];
	PROPSHEETHEADER	psh;

	INT				i;
	TCHAR			szTmp[128] = TEXT( "" );

//<<<FOOBAR>>>
// We need to see if the com port is already open.  If so, then
// when it changes, then close it...
//	if ( hComm != INVALID_HANDLE_VALUE )	{
//		CloseHandle( hComPort );
//		hComPort = INVALID_HANDLE_VALUE;
//	}

//	hComm = hComPort;
//	if ( hComm != INVALID_HANDLE_VALUE )	{
//		wsprintf( szTmp, "%s is still open.\nTo alter settings, port must be closed.\nClose Port Now?",
//			userSavedSettings.szPortInUse );

//		i = MessageBox( hWnd, szTmp, 
//				"Properties", MB_YESNOCANCEL | 
//				MB_ICONERROR | MB_APPLMODAL );

//		if ( i == IDNO || i == IDCANCEL )
//			return 0;

//		CloseHandle( hComm );

//		if ( g_hSendEvent != INVALID_HANDLE_VALUE )	{
//			PulseEvent( g_hSendEvent );
//			Sleep( 100 );
//			CloseHandle( g_hSendEvent );
//			g_hSendEvent = INVALID_HANDLE_VALUE;
//		}
//		hComPort = INVALID_HANDLE_VALUE;
//	}

	memset( &psp, 0, sizeof( psp ));

	for ( i = 0; i < dim( psp ); i++ )	{
		psp[i].dwSize = sizeof( PROPSHEETPAGE );
		psp[i].dwFlags = PSP_DEFAULT;
		psp[i].hInstance = hInst;
		psp[i].lParam = (LPARAM)hWnd;
	}

	// Set the dialog templates for each page...

	psp[0].pszTemplate = MAKEINTRESOURCE( IDD_BASIC_SERIAL_SETUP );
	psp[1].pszTemplate = MAKEINTRESOURCE( IDD_INIT_SERIAL_SETUP );
	psp[2].pszTemplate = MAKEINTRESOURCE( IDD_MESSAGES_SERIAL_SETUP );
	psp[3].pszTemplate = MAKEINTRESOURCE( IDD_ADV_SERIAL_SETUP );

	// Set the Dialog Proc

	psp[0].pfnDlgProc = CommSetupBasicDialogProc;
	psp[1].pfnDlgProc = CommSetupInitsDialogProc;
	psp[2].pfnDlgProc = CommSetupMessagesDialogProc;
	psp[3].pfnDlgProc = CommSetupAdvDialogProc;

	psh.dwSize = sizeof( PROPSHEETHEADER );
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_USEICONID;
	psh.hwndParent = hWnd;
//	psh.hIcon = LoadIcon( hInst, (LPCTSTR)"Kalimba.ico" );
	psh.pszIcon = (LPCTSTR)"Kalimba.ico";
	psh.hInstance = hInst;
	psh.pszCaption = TEXT( "Serial Communications Port Properties" );
	psh.nPages = dim( psp );
	psh.nStartPage = 0;
	psh.ppsp = psp;
	psh.pfnCallback = 0;

	i = PropertySheet( &psh );

	return 0;
}

//
//******************************************************************
//* CommSetupBasicDialogProc
//*
//* Message handler for the Basic Setup Property Sheet...
//* This procedure will only become active if the tab for
//* basic properties has been displayed...
//******************************************************************
//
BOOL CALLBACK CommSetupBasicDialogProc( HWND hWnd, UINT wMsg, WPARAM wParam,
								  LPARAM lParam )	{

	HWND		hwndMain;
	TCHAR		szDev[32] = TEXT( "" );

	INT			i;

	switch ( wMsg )	{

		case WM_INITDIALOG:

			// The generic parameter contains the top-level
			// window handle...

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;
			
			// Save the window handle in the window structure

			SetWindowLong( hWnd, DWL_USER, (LONG)hwndMain );

			// set the initial state of the com port selection
			// and baud rate selection and initialize state of
			// radio buttons.

//			if ( !fCommsPropertiesLoaded )
				SetCommsDefaultBasicProperties( hWnd );

//			else
//				We load from the configuration structure...

			return TRUE;

		case WM_NOTIFY:

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;

			switch ((( NMHDR *)lParam)->code )	{

				//
				// The OK/Apply button was pressed..
				//
				case PSN_APPLY:

					fCommsPropertiesLoaded = TRUE;

					if ( fCommBasicPropertiesChanged )	{

						fCommsPropertiesLoaded = TRUE;
						//
						// Save the Basic Properties to the configuration structure
						//
						
						// Get Com port

						i = SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_GETCURSEL, 0, 0 );

						SendDlgItemMessage( hWnd, IDC_COMBOSERIALPORT, CB_GETLBTEXT, 
							(WPARAM)i, (LPARAM)(LPCSTR)userSavedSettings.szPortInUse );

//						MessageBox( hWnd, userSavedSettings.szPortInUse, "Device", 0 );

						// Get Baud Rate
	
						i = SendDlgItemMessage( hWnd, IDC_B1200, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_1200;

						i = SendDlgItemMessage( hWnd, IDC_B2400, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_2400;

						i = SendDlgItemMessage( hWnd, IDC_B4800, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_4800;

						i = SendDlgItemMessage( hWnd, IDC_B9600, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_9600;

						i = SendDlgItemMessage( hWnd, IDC_B14400, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_14400;

						i = SendDlgItemMessage( hWnd, IDC_B19200, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_19200;

						i = SendDlgItemMessage( hWnd, IDC_B38400, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_38400;

						i = SendDlgItemMessage( hWnd, IDC_B56000, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_56000;

						i = SendDlgItemMessage( hWnd, IDC_B57600, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_57600;

						i = SendDlgItemMessage( hWnd, IDC_B115200, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_115200;

						i = SendDlgItemMessage( hWnd, IDC_B128000, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_128000;

						i = SendDlgItemMessage( hWnd, IDC_B256000, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.BaudRate = CBR_256000;

					// Save Word Size...

						i = SendDlgItemMessage( hWnd, IDC_WORD8, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.ByteSize = 8;

						i = SendDlgItemMessage( hWnd, IDC_WORD7, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.ByteSize = 7;

						i = SendDlgItemMessage( hWnd, IDC_WORD6, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.ByteSize = 6;

						i = SendDlgItemMessage( hWnd, IDC_WORD5, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.ByteSize = 5;

						i = SendDlgItemMessage( hWnd, IDC_WORD4, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.ByteSize = 4;

					// Save parity 

						i = SendDlgItemMessage( hWnd, IDC_PARITY_NONE, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.Parity = NOPARITY;

						i = SendDlgItemMessage( hWnd, IDC_PARITY_EVEN, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.Parity = EVENPARITY;

						i = SendDlgItemMessage( hWnd, IDC_PARITY_ODD, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.Parity = ODDPARITY;

						i = SendDlgItemMessage( hWnd, IDC_PARITY_MARK, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.Parity = MARKPARITY;

						i = SendDlgItemMessage( hWnd, IDC_PARITY_SPACE, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.Parity = SPACEPARITY;

					// Save Stop Bits

						i = SendDlgItemMessage( hWnd, IDC_STOPBITS1, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.StopBits = ONESTOPBIT;

						i = SendDlgItemMessage( hWnd, IDC_STOPBITS15, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.StopBits = ONE5STOPBITS;

						i = SendDlgItemMessage( hWnd, IDC_STOPBITS2, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.dcb.StopBits = TWOSTOPBITS;

						
						return FALSE;
					}
					return FALSE;

				//
				// The cancel button was pressed.
				//
				case PSN_QUERYCANCEL:

					if ( fCommBasicPropertiesChanged )	{

						i = MessageBox( hWnd, 
							"Communications Properties you changed will not be saved!\n\nDo you still wish to abort?", 
							"Properties", 
							MB_YESNO | MB_APPLMODAL | MB_ICONQUESTION );

						if ( i == IDYES )	{
							fCommBasicPropertiesChanged = FALSE;
							return TRUE;
						}
						else	{

							return FALSE;
						}
					}
					return TRUE;
			}
			return FALSE;


		case WM_COMMAND:

			// First see if someone fooled around with the combo box
			// for com ports...

			switch( HIWORD( wParam ))	{

				case CBN_SELCHANGE:

					fCommBasicPropertiesChanged = TRUE;

//					MessageBox( hWnd, "ComboBox...", "Drop", 0 );
					return TRUE;
			}

			switch( LOWORD( wParam ))	{

				case IDOK:

					// Get all the new Parameters and store them...

				case IDCANCEL:

					// We are out of here...

					MessageBox( hWnd, "Got OK or Cancel...", "Button", 0 );
					EndDialog( hWnd, 0 );
					return TRUE;

//				case IDC_COMBOSERIALPORT:
//					MessageBox( hWnd, "ComboBox...", "Drop", 0 );
//					fCommBasicPropertiesChanged = TRUE;
//					return TRUE;

				case IDC_B256000:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B128000:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B115200:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B57600:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B56000:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B38400:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B19200:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B14400:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B9600:

					fCommBasicPropertiesChanged = TRUE;

//					SendDlgItemMessage( hWnd, 
//							PSN_APPLY, WM_ENABLE, 0, 0 );

					return TRUE;

				case IDC_B4800:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B2400:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_B1200:

					fCommBasicPropertiesChanged = TRUE;
					return TRUE;

				case IDC_BASIC_SERIAL_SETDEFAULT:

					MessageBox( hWnd, "Are you sure?", "Reset", 0 );
					return TRUE;
			}
		break;
	}
	return FALSE;
}

//
//******************************************************************
//* CommSetupInitsDialogProc
//*
//* Message handler for the Inits Setup on the property page...
//******************************************************************
//
BOOL CALLBACK CommSetupInitsDialogProc( HWND hWnd, UINT wMsg, WPARAM wParam,
								  LPARAM lParam )	{

	HWND		hwndMain;
	TCHAR		szDev[60] = TEXT("");
	INT			i;

	switch ( wMsg )	{

		case WM_INITDIALOG:

			// The generic parameter contains the top-level
			// window handle...

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;
			
			// Save the window handle in the window structure

			SetWindowLong( hWnd, DWL_USER, (LONG)hwndMain );

			// These are initial values...Should be replaced by
			// reading the config information from file...

//			if ( !fCommsPropertiesLoaded )

				SetCommsDefaultInitsProperties( hWnd );

//			else
//				We Load from the configuration Structure...


			return TRUE;

		case WM_NOTIFY:

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;

			switch ((( NMHDR *)lParam)->code )	{

				case PSN_APPLY:

					if ( fCommInitsPropertiesChanged )	{

						// Save the escape Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_ESCAPE_CODE, 
								EM_LINELENGTH, (WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_ESCAPE_CODE, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szEscapeCode );
						
						// Save the OnHook Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_ONHOOK, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_ONHOOK, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szOnHook );

						// Save the OffHook Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_OFFHOOK, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_OFFHOOK, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szOffHook );

						// Save the ReturnOnline Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_RETURNONLINE, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_RETURNONLINE, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szReturnOnline );

						// Svae the Attention Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_ATTENTION, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_ATTENTION, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szAttention );

						// Save the Dial Prefix Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_DIALPREFIX, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_DIALPREFIX, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szDialPrefix );

						// Save the Reset Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_RESET, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_RESET, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szReset );

						// Save the Init1 Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_INIT1, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_INIT1, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szInit1 );

						// Save the Init2 Code
						i = SendDlgItemMessage( hWnd, IDC_INIT_INIT2, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_INIT_INIT2, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szInit2 );

						if ( SendDlgItemMessage( hWnd, IDC_INIT_CR_YES, 
												BM_GETSTATE, 0, 0 ) == BST_CHECKED )
							userSavedSettings.fWantsCr = TRUE;
						else
							userSavedSettings.fWantsCr = FALSE;

//						MessageBox( hWnd, userSavedSettings.szInit2, "Init2 Code", 0 );
					}

					return FALSE;

				case PSN_QUERYCANCEL:

					if ( fCommInitsPropertiesChanged )	{
						MessageBox( hWnd, "Exit with changed Init props...", "Button", 0 );						
					}
					return TRUE;
					//
					// Go Save the properties
			}
			return FALSE;


		case WM_COMMAND:

			switch( HIWORD( wParam ))	{

				case EN_CHANGE:

					fCommInitsPropertiesChanged = TRUE;

//					MessageBox( hWnd, "Changed...", "Edit", 0 );
					return TRUE;

			}

			switch( LOWORD( wParam ))	{
				case IDOK:

					// Get all the new Parameters and store them...

				case IDCANCEL:
					// We are out of here...

					EndDialog( hWnd, 0 );
					return TRUE;

				case IDC_INIT_CR_YES:

					fCommInitsPropertiesChanged = TRUE;

//					MessageBox( hWnd, "Changed..CheckBox.", "Init", 0 );
					return TRUE;
			}
		break;
	}
	return FALSE;
}

//
//******************************************************************
//* CommSetupMessagesDialogProc
//*
//* Message handler for the CommSetupMessagesDialogBox...
//******************************************************************
//
BOOL CALLBACK CommSetupMessagesDialogProc( HWND hWnd, UINT wMsg, WPARAM wParam,
								  LPARAM lParam )	{

	HWND		hwndMain;
	TCHAR		szDev[60] = TEXT("");
	INT			i;

	switch ( wMsg )	{

		case WM_INITDIALOG:

			// The generic parameter contains the top-level
			// window handle...

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;
			
			// Save the window handle in the window structure

			SetWindowLong( hWnd, DWL_USER, (LONG)hwndMain );

			// Place initial values into the fields for this property
			// sheet...
//			if ( !fCommsPropertiesLoaded )
				SetCommsDefaultMessageProperties( hWnd );
//			else
//				We need to load values from the configuration
//				structure...


			return TRUE;

		case WM_NOTIFY:

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;

			switch ((( NMHDR *)lParam)->code )	{

				case PSN_APPLY:

//					MessageBox( hWnd, "Got Messages PSN_APPLY...", "Button", 0 );

					if ( fCommMessagesPropertiesChanged )	{

						// Save Msg OK Code
						i = SendDlgItemMessage( hWnd, IDC_MSG_OK, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_OK, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgOk );

						//Save Msg BUSY Code
						i = SendDlgItemMessage( hWnd, IDC_MSG_BUSY, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_BUSY, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgBusy );

						//Save Msg RING Code
						i = SendDlgItemMessage( hWnd, IDC_MSG_RING, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_RING, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgRing );

						// Save Msg VOICE Code

						i = SendDlgItemMessage( hWnd, IDC_MSG_VOICE, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_VOICE, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgVoice );

						//Save Msg ERROR Code
						i = SendDlgItemMessage( hWnd, IDC_MSG_ERROR, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_ERROR, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgError );

						// Save Msg NOCARRIER
						i = SendDlgItemMessage( hWnd, IDC_MSG_CARRIER, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_CARRIER, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgNoCarrier );

						//Save Msg NODIALTONE Code
						i = SendDlgItemMessage( hWnd, IDC_MSG_DIALTONE, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_DIALTONE, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgNoDialTone );

						// Save Msg NOANSWER Code
						i = SendDlgItemMessage( hWnd, IDC_MSG_ANSWER, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_ANSWER, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgNoAnswer );

						// Save the Connection Messages

						// 1200
						i = SendDlgItemMessage( hWnd, IDC_MSG_B1200, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B1200, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect1200 );

						// 2400
						i = SendDlgItemMessage( hWnd, IDC_MSG_B2400, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B2400, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect2400 );

						// 4800
						i = SendDlgItemMessage( hWnd, IDC_MSG_B4800, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B4800, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect4800 );

						// 9600
						i = SendDlgItemMessage( hWnd, IDC_MSG_B9600, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B9600, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect9600 );

						// 14400
						i = SendDlgItemMessage( hWnd, IDC_MSG_B14400, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B14400, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect14400 );

						// 19200
						i = SendDlgItemMessage( hWnd, IDC_MSG_B19200, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B19200, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect19200 );

						// 38400
						i = SendDlgItemMessage( hWnd, IDC_MSG_B38400, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B38400, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect38400 );

						// 56000
						i = SendDlgItemMessage( hWnd, IDC_MSG_B56000, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B56000, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect56000 );

						// 57600
						i = SendDlgItemMessage( hWnd, IDC_MSG_B57600, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B57600, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect57600 );

						// 115200
						i = SendDlgItemMessage( hWnd, IDC_MSG_B115200, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B115200, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect115200 );

						// 128000
						i = SendDlgItemMessage( hWnd, IDC_MSG_B128000, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B128000, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect128000 );

						// 256000
						i = SendDlgItemMessage( hWnd, IDC_MSG_B256000, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;
						SendDlgItemMessage( hWnd, IDC_MSG_B256000, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)userSavedSettings.szMsgConnect256000 );

					}

					return FALSE;

				case PSN_QUERYCANCEL:

					if ( fCommMessagesPropertiesChanged )	{
						MessageBox( hWnd, "Exit with changed Message props...", "Button", 0 );						
					}
					return TRUE;
					//
					// Go Save the properties
			}
			return FALSE;


		case WM_COMMAND:

			switch( HIWORD( wParam ))	{

				case EN_CHANGE:

					fCommMessagesPropertiesChanged = TRUE;

//					MessageBox( hWnd, "Changed...", "Edit", 0 );
					return TRUE;

			}

			switch( LOWORD( wParam ))	{
				case IDOK:

					// Get all the new Parameters and store them...

				case IDCANCEL:
					// We are out of here...
					EndDialog( hWnd, 0 );
					return TRUE;
			}
		break;
	}
	return FALSE;
}

//
//******************************************************************
//* CommSetupAdvDialogProc
//*
//* Message handler for the CommSetupAdvDialogBox...
//******************************************************************
//
BOOL CALLBACK CommSetupAdvDialogProc( HWND hWnd, UINT wMsg, WPARAM wParam,
								  LPARAM lParam )	{

	HWND		hwndMain;
	INT			i;
	TCHAR		szTmp[20] = TEXT( "" );

	switch ( wMsg )	{

		case WM_INITDIALOG:

			// The generic parameter contains the top-level
			// window handle...

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;
			
			// Save the window handle in the window structure

			SetWindowLong( hWnd, DWL_USER, (LONG)hwndMain );

			SetCommsDefaultAdvancedProperties( hWnd );

			return TRUE;

		case WM_NOTIFY:

			hwndMain = (HWND)((LPPROPSHEETPAGE)lParam)->lParam;

			switch ((( NMHDR *)lParam)->code )	{

				case PSN_APPLY:

					fCommsPropertiesLoaded = TRUE;

					if ( fCommAdvPropertiesChanged )	{
						i = SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS, CB_GETCURSEL, 0, 0 );
						SendDlgItemMessage( hWnd, IDC_COMBO_PORT_ADDRESS, CB_GETLBTEXT, (WPARAM)i,
										(LPARAM)(LPCSTR)userSavedSettings.szPortAddress );

						i = SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_GETCURSEL, 0, 0 );
						SendDlgItemMessage( hWnd, IDC_COMBO_IRQ_LINE, CB_GETLBTEXT, (WPARAM)i,
										(LPARAM)(LPCSTR)userSavedSettings.szPortIRQ );

						i = SendDlgItemMessage( hWnd, IDC_COMBO_INTERRUPT, CB_GETCURSEL, 0, 0 );

						SendDlgItemMessage( hWnd, IDC_COMBO_INTERRUPT, CB_GETLBTEXT, (WPARAM)i,
										(LPARAM)(LPCSTR)userSavedSettings.szPortInterrupt );
	
						i = SendDlgItemMessage( hWnd, IDC_ADV_HSRTS, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.fRTSControl = TRUE;
						else
							userSavedSettings.fRTSControl = FALSE;

						i = SendDlgItemMessage( hWnd, IDC_ADV_HSDTR, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.fDTRControl = TRUE;
						else
							userSavedSettings.fDTRControl = FALSE;

						i = SendDlgItemMessage( hWnd, IDC_ADV_HSDSR, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.fDSRControl = TRUE;
						else
							userSavedSettings.fDSRControl = FALSE;

						i = SendDlgItemMessage( hWnd, IDC_ADV_HSCTS, BM_GETCHECK, 0, 0 );
						if ( i )
							userSavedSettings.fCTSControl = TRUE;
						else
							userSavedSettings.fCTSControl = FALSE;

						// Queue Sizes

						i = SendDlgItemMessage( hWnd, IDC_ADV_RCV_QUEUE, EM_LINELENGTH,
											(WPARAM)0, 0 );

						i++;
						SendDlgItemMessage( hWnd, IDC_ADV_RCV_QUEUE, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)szTmp );

//						MessageBox( hWnd, szTmp, "Queue", 0 );

						userSavedSettings.dwInQueueSize = (DWORD)atol( szTmp );

						i = SendDlgItemMessage( hWnd, IDC_ADV_XMIT_QUEUE, EM_LINELENGTH,
											(WPARAM)0, 0 );
						i++;

						SendDlgItemMessage( hWnd, IDC_ADV_XMIT_QUEUE, WM_GETTEXT, (WPARAM)i,
											(LPARAM)(LPCSTR)szTmp );

						userSavedSettings.dwOutQueueSize = (DWORD)atol( szTmp );
					}
					return FALSE;

				case PSN_QUERYCANCEL:

					if ( fCommAdvPropertiesChanged )	{
						MessageBox( hWnd, "Exit with changed Advanced props...", "Button", 0 );						
					}
					return TRUE;

			}
			return FALSE;

		case WM_COMMAND:

			switch( HIWORD( wParam ))	{

				case CBN_SELCHANGE:

					fCommAdvPropertiesChanged = TRUE;

					switch( LOWORD( wParam ))	{

						case IDC_COMBO_PORT_ADDRESS:

//							MessageBox( hWnd, "Port Address...", "Drop", 0 );
							break;

						case IDC_COMBO_IRQ_LINE:
							// Update userSavedSettings
//							MessageBox( hWnd, "Port IRQ...", "Drop", 0 );
							break;

						case IDC_COMBO_INTERRUPT:
							// Update userSavedSettings
//							MessageBox( hWnd, "Port Interrupt...", "Drop", 0 );
							break;

					}
					return TRUE;

				case EN_CHANGE:

					fCommAdvPropertiesChanged = TRUE;

					switch( LOWORD( wParam ))	{

						case IDC_ADV_XMIT_QUEUE:
							break;

						case IDC_ADV_RCV_QUEUE:
							break;
					}
					return TRUE;
			}

			switch( LOWORD( wParam ))	{

				case IDOK:

					// Get all the new Parameters and store them...

				case IDCANCEL:
					// We are out of here...
					EndDialog( hWnd, 0 );
					return TRUE;

					// user fooled with the check box
				case IDC_ADV_LOWER_DTR:

					fCommAdvPropertiesChanged = TRUE;
					return TRUE;

					// user fooled with the check box
				case IDC_ADV_LOCK_BAUD:

					fCommAdvPropertiesChanged = TRUE;
					return TRUE;

			}
		break;
	}
	return FALSE;
}

//*
//* End of Setup.c
//*