
//*
//******************************************************************
//*
//*	Module:			TTYInfo.c
//*
//*	Purpose:		To handle all aspects of reading and writing to 
//*					TTY Display Window.
//*
//*	Date Written:	December 22nd, 1999
//*
//******************************************************************
//*

#include <windows.h>
#include "Resource.h"
#include "Globals.h"
#include "TTYInfo.h"

char	szChar[100];
char	CURRPT[10];
char	buff[256];
int		VTc, VTr, SVTc, SVTr, VTwt, VTwb, VTa;

void OpenTTYWindow( HWND hWnd );
void CloseTTYWindow( void );
BOOL NEAR ScrollTTYVert( HWND hWnd, WORD wScrollCmd, WORD wScrollPos );
BOOL NEAR ScrollTTYHorz( HWND hWnd, WORD wScrollCmd, WORD wScrollPos );
BOOL NEAR PaintTTY( HWND hWnd );
BOOL NEAR MoveTTYCursor( HWND hWnd );
BOOL NEAR SetTTYFocus( HWND hWnd );
BOOL NEAR KillTTYFocus( HWND hWnd );
BOOL NEAR SizeTTY( HWND hwnd, WORD wWidth, WORD wHeight );
BOOL WriterAddNewNode( DWORD, DWORD, char, char *, HANDLE, HWND, HWND );

int WINAPI TTYChildProc( HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam );

void OutputABufferToWindow( HWND hTTY, char * lpBuf, DWORD dwBufLen );
void OutputABufferToFile( HANDLE hFile, char * lpBuf, DWORD dwBufLen );
void OutputABuffer( HWND hTTY, char * lpBuf, DWORD dwBufLen );

//*
//* Video Emulation ruotines
//*

void HandleTTYEmulation( HWND hTTY, int cChar );
void HandleVT100Emulation( HWND hTTY, int cChar );
void HandleAnsiEmulation( HWND hTTY, int cChar );

void DisplayScreenChar( HWND hTTY, int cChar );

//*
//******************************************************************
//*
//******************************************************************
//*

void OpenTTYWindow( HWND hWnd )	{

//	WORD	wTop;
//	WORD	wHeight;
//	WORD	wWidth = 200;
//	char	szBuf[512];

	lstrcpy( szChar, "\0" );

	if ( TTYWINDOWISOPEN( TTYInfo ))	{
		CloseTTYWindow();
		return;
	}

	if ( ghWndTTY != NULL )	{
		ShowWindow( ghWndTTY, CS_SHOW );
		return;
	}


//	MessageBox( hWnd, "Selected", "TTY", MB_OK );
//	ghWndTTY = CreateWindowEx( WS_EX_CLIENTEDGE, "KalimbaTTYWinClass", "TTY Window",
//					WS_CHILD | WS_VISIBLE | WS_VSCROLL | DS_3DLOOK,
//					0,0,
//					0,0,
//					hWnd, (HMENU)ID_TTYWINDOW, hInst, NULL );

	FillMemory( SCREEN( TTYInfo ), MAXROWS * MAXCOLS, ' ' );

	LOCALECHO( TTYInfo ) = TRUE;
	NEWLINE( TTYInfo ) = TRUE;
	AUTOWRAP( TTYInfo ) = TRUE;

	// Set Default Emulation mode; simple TTY display...

	EMULATION( TTYInfo ) = EMU_VT100;

	ghWndTTY = CreateWindow( "KalimbaTTYWinClass", "TTY Data",
//					WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_THICKFRAME,
					WS_CHILD | WS_VSCROLL | WS_THICKFRAME,
					0,0,
					0,0,
					hWnd, (HMENU)ID_TTYWINDOW, hInst, NULL );

	if ( ghWndTTY == NULL )
		MessageBox( hWnd, "Error Creating TTY Window", "Fatal", 0 );
	else	{
//		MessageBox( hWnd, "Created", "TTY", MB_OK );
//		ShowWindow( ghWndTTY, SW_SHOW | SW_SHOWNORMAL | SW_MAXIMIZE );
//		UpdateWindow( ghWndTTY ) ;		
//		TTYWINDOWISOPEN( TTYInfo ) = TRUE;
	
//		wHeight = 200 - (( STATUSFACTOR + SETTINGSFACTOR ) * gwBaseY );
//		wTop = SETTINGSFACTOR * gwBaseY;
//		wsprintf( szBuf, "h=%d, w=%d, gwBaseY=%d", wHeight, wWidth, gwBaseY );
//		MessageBox( hWnd, szBuf, "Params", 0 );

//		MoveWindow( ghWndTTY, 0, wTop, wWidth, wHeight, TRUE );
	}
	return;
}

//*
//******************************************************************
//*
//******************************************************************
//*

void CloseTTYWindow( void )	{

//	if ( !TTYWINDOWISOPEN( TTYInfo ))
//		return;

	ShowWindow( ghWndTTY, CS_HIDE );

//	DestroyWindow(ghWndTTY);
	TTYWINDOWISOPEN( TTYInfo ) = FALSE;
}

//*
//******************************************************************
//*
//******************************************************************
//*

BOOL NEAR ScrollTTYVert( HWND hWnd, WORD wScrollCmd, WORD wScrollPos )	{

	int		nScrollAmt;

	switch( wScrollCmd)	{

		case SB_TOP:

			nScrollAmt = -YOFFSET( TTYInfo );
			break;

		case SB_BOTTOM:

			nScrollAmt = -YOFFSET( TTYInfo ) - YOFFSET( TTYInfo );
			break;

		case SB_PAGEUP:

			nScrollAmt = -YSIZE( TTYInfo );
			break;

		case SB_PAGEDOWN:

			nScrollAmt = YSIZE( TTYInfo );
			break;

		case SB_LINEUP:

			nScrollAmt = -YCHAR( TTYInfo );
			break;

		case SB_LINEDOWN:

			nScrollAmt = YCHAR( TTYInfo );
			break;

		case SB_THUMBPOSITION:

			nScrollAmt = wScrollPos - YOFFSET( TTYInfo );
			break;

		default:

			return ( FALSE );

	}

	if ( ( YOFFSET( TTYInfo ) + nScrollAmt ) > YSCROLL( TTYInfo ))
		nScrollAmt = YSCROLL( TTYInfo ) - YOFFSET( TTYInfo );


	if ( ( YOFFSET( TTYInfo ) + nScrollAmt ) < 0 )
		nScrollAmt = -YOFFSET( TTYInfo );

	ScrollWindowEx( hWnd, 0, -nScrollAmt, NULL, NULL, NULL, NULL, SW_INVALIDATE | SW_ERASE );

	YOFFSET( TTYInfo ) = YOFFSET( TTYInfo ) + nScrollAmt;
	SetScrollPos( hWnd, SB_VERT, YOFFSET( TTYInfo ), TRUE );

	return( TRUE );
}


//*
//******************************************************************
//*
//******************************************************************
//*

BOOL NEAR ScrollTTYHorz( HWND hWnd, WORD wScrollCmd, WORD wScrollPos )	{


	int		nScrollAmt;

	switch( wScrollCmd)	{

		case SB_TOP:

			nScrollAmt = -XOFFSET( TTYInfo );
			break;

		case SB_BOTTOM:

			nScrollAmt = -XOFFSET( TTYInfo ) - XOFFSET( TTYInfo );
			break;

		case SB_PAGEUP:

			nScrollAmt = -XSIZE( TTYInfo );
			break;

		case SB_PAGEDOWN:

			nScrollAmt = XSIZE( TTYInfo );
			break;

		case SB_LINEUP:

			nScrollAmt = -XCHAR( TTYInfo );
			break;

		case SB_LINEDOWN:

			nScrollAmt = XCHAR( TTYInfo );
			break;

		case SB_THUMBPOSITION:

			nScrollAmt = wScrollPos - XOFFSET( TTYInfo );
			break;

		default:

			return ( FALSE );

	}

	if (( XOFFSET( TTYInfo ) + nScrollAmt ) > XSCROLL( TTYInfo ))
		nScrollAmt = XSCROLL( TTYInfo ) - XOFFSET( TTYInfo );

	if (( XOFFSET( TTYInfo ) + nScrollAmt ) < 0 )
		nScrollAmt = -XOFFSET( TTYInfo );

	ScrollWindowEx( hWnd, -nScrollAmt, 0, NULL, NULL, NULL, NULL, SW_INVALIDATE | SW_ERASE );

	XOFFSET( TTYInfo ) = XOFFSET( TTYInfo ) + nScrollAmt;

	SetScrollPos( hWnd, SB_HORZ, XOFFSET( TTYInfo ), TRUE );

	return ( TRUE );

}


//*
//******************************************************************
//*
//******************************************************************
//*

BOOL NEAR PaintTTY( HWND hWnd )	{

	PAINTSTRUCT			ps;
	HFONT				hOldFont;
	RECT				rect;
	HDC					hDC;
	int					nRow, nCol, nEndRow, nEndCol;
	int					nCount, nHorzPos, nVertPos;


	hDC = BeginPaint( hWnd, &ps );
	hOldFont = SelectObject( hDC, HTTYFONT( TTYInfo ));
	SetTextColor( hDC, FGCOLOR( TTYInfo ));
	SetBkColor( hDC, GetSysColor( COLOR_WINDOW ));
//	SetBkColor( hDC, GetSysColor( COLOR_BACKGROUND ));

	rect = ps.rcPaint;
	nRow =
		min( MAXROWS - 1,
				max( 0, ( rect.top + YOFFSET( TTYInfo )) / YCHAR( TTYInfo )));

	nEndRow =
		min( MAXROWS - 1,
				(( rect.bottom + YOFFSET( TTYInfo ) - 1 ) / YCHAR( TTYInfo )));

	nCol =
		min( MAXROWS - 1,
				max( 0, ( rect.left + XOFFSET( TTYInfo )) / XCHAR( TTYInfo )));

	nEndCol =
		min( MAXCOLS - 1,
				(( rect.right + XOFFSET( TTYInfo ) - 1 ) / XCHAR( TTYInfo )));

	nCount = nEndCol - nCol + 1;
	
	for ( ; nRow <= nEndRow; nRow++ )	{

		nVertPos = ( nRow * YCHAR( TTYInfo )) - YOFFSET( TTYInfo );
		nHorzPos = ( nCol * XCHAR( TTYInfo )) - XOFFSET( TTYInfo );
		rect.top = nVertPos;
		rect.bottom = nVertPos + YCHAR( TTYInfo );
		rect.left = nHorzPos;
		rect.right = nHorzPos + XCHAR( TTYInfo ) * nCount;

		SetBkMode( hDC, OPAQUE );
		ExtTextOut( hDC, nHorzPos, nVertPos, ETO_OPAQUE | ETO_CLIPPED, &rect,
					( LPSTR )( SCREEN( TTYInfo ) + nRow * MAXCOLS + nCol ),
					nCount, NULL );
	}

	SelectObject( hDC, hOldFont );
	EndPaint( hWnd, &ps );
	MoveTTYCursor( hWnd );
	return( TRUE );

} // End of PaintTTY()


//*
//******************************************************************
//*
//******************************************************************
//*

BOOL NEAR MoveTTYCursor( HWND hWnd )	{

	if ( CONNECTED( TTYInfo ) && ( CURSORSTATE( TTYInfo ) & CS_SHOW ))
		SetCaretPos( (COLUMN( TTYInfo ) * XCHAR( TTYInfo )) -
						XOFFSET( TTYInfo ),
						( ROW( TTYInfo ) * YCHAR( TTYInfo )) -
						YOFFSET( TTYInfo ));

	return( TRUE );

} // End of MoveTTYCursor()

//*
//******************************************************************
//*
//******************************************************************
//*

BOOL NEAR SetTTYFocus( HWND hWnd )	{

	if ( CONNECTED( TTYInfo ) && ( CURSORSTATE( TTYInfo ) != CS_SHOW ))	{

		CreateCaret( hWnd, NULL, XCHAR( TTYInfo ), YCHAR( TTYInfo ));
		ShowCaret( hWnd );
		CURSORSTATE( TTYInfo ) = CS_SHOW;
	}

	MoveTTYCursor( hWnd );
	return( TRUE );

} // End of SetTTYFocus()


//*
//******************************************************************
//*
//******************************************************************
//*


BOOL NEAR KillTTYFocus( HWND hWnd )	{

	if ( CURSORSTATE( TTYInfo ) != CS_HIDE )	{

		HideCaret( hWnd );
		DestroyCaret();
		CURSORSTATE( TTYInfo ) = CS_HIDE;
	}
	return( TRUE );

} // end of KillTTYFocus()

//*
//******************************************************************
//*
//******************************************************************
//*

BOOL NEAR SizeTTY( HWND hWnd, WORD wWidth, WORD wHeight )	{

	int nScrollAmt;

	YSIZE( TTYInfo ) = (int) wHeight;
	YSCROLL( TTYInfo ) = max( 0, ( MAXROWS * YCHAR( TTYInfo )) - 
								YSIZE( TTYInfo ));

	nScrollAmt = min( YSCROLL( TTYInfo ), YOFFSET( TTYInfo )) -
						YOFFSET( TTYInfo );

	ScrollWindow( hWnd, 0, -nScrollAmt, NULL, NULL );

	YOFFSET( TTYInfo ) = YOFFSET( TTYInfo ) + nScrollAmt;

	SetScrollPos( hWnd, SB_VERT, YOFFSET( TTYInfo ), FALSE );
	SetScrollRange( hWnd,SB_VERT, 0, YSCROLL( TTYInfo ), TRUE );

	//
	//
	//

	XSIZE( TTYInfo ) = (int) wHeight;
	XSCROLL( TTYInfo ) = max( 0, ( MAXCOLS * XCHAR( TTYInfo )) -
							XSIZE( TTYInfo ));

	nScrollAmt = min( XSCROLL( TTYInfo ), XOFFSET( TTYInfo )) -
					XOFFSET( TTYInfo );

	ScrollWindow( hWnd, nScrollAmt, 0, NULL, NULL );
	XOFFSET( TTYInfo ) = XOFFSET( TTYInfo ) + nScrollAmt;
	SetScrollRange( hWnd, SB_HORZ, 0, XSCROLL( TTYInfo ), FALSE );

	SetScrollPos( hWnd, SB_HORZ, XOFFSET( TTYInfo ), TRUE );

	InvalidateRect( hWnd, NULL, FALSE );

	return( TRUE );

} // end of SizeTTY()


//*
//******************************************************************
//*
//******************************************************************
//*

int WINAPI TTYChildProc( HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam )	{

	switch( uMessage )	{

		case WM_VSCROLL:

			ScrollTTYVert( hWnd, LOWORD( wParam ), HIWORD( wParam ));
			break;

		case WM_HSCROLL:
			
			ScrollTTYHorz( hWnd, LOWORD( wParam ), HIWORD( wParam ));
			break;

		case WM_SIZE:

			SizeTTY( hWnd, LOWORD( lParam ), HIWORD( lParam ));
			break;

		case WM_PAINT:

			PaintTTY( hWnd );
			break;

		case WM_CHAR:

			{
				//
				//
				//

				if ( CONNECTED( TTYInfo ))	{

					szChar[0] = ( char )wParam;

					//
					// Need a PermissionToWrite Flag here
					//

					if ( !WriterAddNewNode( WRITE_CHAR, 1, ( char ) wParam, NULL, NULL, NULL, hWnd ))
						return FALSE;

//					if ( LOCALECHO( TTYInfo ))
						OutputABufferToWindow( ghWndTTY, ( CHAR * ) &wParam, 1 );

				}
			}

			break;

		case WM_SETFOCUS:

			SetTTYFocus( ghWndTTY );
			break;

		case WM_KILLFOCUS:

			KillTTYFocus( ghWndTTY );
			break;

		case WM_MOUSEACTIVATE:
			
			SetFocus( hWnd );
			return MA_ACTIVATE;
			break;

		default:

			return DefWindowProc( hWnd, uMessage, wParam, lParam );
	}

	return 0L;
}


//*
//******************************************************************
//*
//* OutputABufferToWindow
//*
//* Routine to handle incoming data off the serial port and handle
//* the Emulations if needed...
//* 
//* Emulations: TTY, VT100, ANSI
//*
//* Parameters:
//*		HWND			-
//*		char *			-
//*		DWORD			-
//*
//* Returns:
//*		Nothing.
//*
//******************************************************************
//*

void OutputABufferToWindow( HWND hTTY, char * lpBuf, DWORD dwBufLen )	{

//	RECT	rect;
	int		i;

	for ( i = 0; i < ( int )dwBufLen; i++ )	{

		switch( EMULATION( TTYInfo ))	{

			case EMU_TTY:
			case EMU_ANSI:

				HandleTTYEmulation( hTTY, (int)lpBuf[i] );
				break;

			case EMU_VT100:

				HandleVT100Emulation( hTTY, (int)lpBuf[i] );
				break;

			default:
				break;
		}

	}

	MoveTTYCursor( hTTY );
	return;
}

//*
//******************************************************************
//*
//* HandleTTYEmulation
//*
//* Routine to handle Simple Chars that need to be ouputted to
//* the TTY Window
//* 
//* Parameters:
//*		HWND			-  Handle of the calling window
//*		int				-  Character to write
//*
//* Returns:
//*		Nothing.
//*
//******************************************************************
//*


void HandleTTYEmulation( HWND hTTY, int cChar )	{

//	RECT	rect;

	cChar &= 0xff;

	switch( cChar )	{

		case 0:		// NULL characters... blah!

			cChar |= 0x40;
			DisplayScreenChar( hTTY, cChar );

			break;

		case ASCII_BEL:		// Ding!

			MessageBeep( 0 );
			break;

		case ASCII_BS:		// Oooops!

			if ( COLUMN( TTYInfo ) > 0 )
				COLUMN( TTYInfo )-- ;
			else
				MessageBeep( 0 );
			break;

		case ASCII_CR:

			COLUMN( TTYInfo ) = 0;
			if ( !NEWLINE( TTYInfo ))
				break;


			//Fall through

		case ASCII_LF:

			if ( ROW( TTYInfo )++ == MAXROWS - 1 )	{

				MoveMemory( (LPSTR) ( SCREEN( TTYInfo )),
						(LPSTR) ( SCREEN( TTYInfo ) + MAXCOLS ),
						( MAXROWS - 1 ) * MAXCOLS );

				FillMemory( (LPSTR) ( SCREEN( TTYInfo ) + ( MAXROWS - 1 ) * MAXCOLS ),
					MAXCOLS, ' ' );

				InvalidateRect( hTTY, NULL, FALSE );
				ROW( TTYInfo )--;

			}
			break;

		default:

			DisplayScreenChar( hTTY, cChar );

			break;
	}
}


//*
//******************************************************************
//*
//* HandleVT100Emulation
//*
//* Routine to handle standard VT100 Emulation.  Interprets screen
//* formatting commands, and color attributes.
//* 
//* Parameters:
//*		HWND			-  Handle of the calling window
//*		int				-  Character to write
//*
//* Returns:
//*		Nothing.
//*
//******************************************************************
//*

void HandleVT100Emulation( HWND hTTY, int cChar )	{

//	RECT	rect;
	static	int		TFlag = 0;
	static	int		TFlag1 = 0;
	static	int		TFp[10];
	static	int		PrtFlag = 0;

	static	int		i;
	static	int		j;

	cChar &= 0x00ff;

	switch( TFlag )	{

		case 0:

			if ( cChar == 0x1b )	{		// Setting up for Something Special

				//
				// Clear out the command buffer...
				// Set up for next character to be interpreted as
				// part of the special command

				for ( TFlag1 = 0; TFlag1 < 10; TFlag1++ )
					TFp[TFlag1] = 0;
				TFlag = 1;
				TFlag1 = 0;
				return;
			}

			//
			// Looks like we got a normal character in
			//
			switch( cChar )	{

				case 0:
				case 1:
				case 2:
				case 3:
				case 4:
				case 5:
				case 6:

					DisplayScreenChar( hTTY, cChar + '@' );
					break;

				case 7:

					HandleTTYEmulation( hTTY, ASCII_BEL );
					break;

				case 8:

					HandleTTYEmulation( hTTY, ASCII_BS );
					break;

				case 9:

					DisplayScreenChar( hTTY, cChar + '@' );
					break;

				case 10:

					HandleTTYEmulation( hTTY, ASCII_LF );
					break;

				case 11:
				case 12:

					DisplayScreenChar( hTTY, cChar + '@' );
					break;

				case 13:

					HandleTTYEmulation( hTTY, ASCII_CR );
					break;

				case 14:
				case 15:
				case 16:
				case 17:
				case 18:
				case 19:
				case 20:
				case 21:
				case 22:
				case 23:
				case 24:
				case 25:
				case 26:
				case 27:
				case 28:
				case 29:
				case 30:
				case 31:
					break;

				default:

					DisplayScreenChar( hTTY, cChar );

					break;
			}
			break;

		case 1:

			switch( cChar )	{

				case 24:		// Cancel sequence
				case 26:

					TFlag = 0;
					break;

				case 0x1b:
					return;

				case '[':

					TFlag = 2;
					TFlag1 = 0;
					return;

				case '|':
					break;

				case '(':
				case ')':

					TFlag = 3;
					return;

				case 0x12:
					TFlag = 0;
					break;

				case 0x0d:
					HandleTTYEmulation( hTTY, ASCII_CR );
					break;

				case 'D':	// Index
				case 'E":	// Next Line

					break;

				case 'F':	/// Start of selected area
				case 'G':	// End of selected area
				case 'H':	// Horiz. Tab set
				case 'L':	// Partial line down
				case 'K'	// Partial line up
				case 'M':	// Reverse Index
				case 'U':	// Message waiting
				case 'V':	// Start of protected area
				case 'W':	// End of protect area
				case 'Z':	//
				case 'c':	//
				case '=':	//
				case '>':	//
				case '7':	//	Save screen attributes?
				case '8':	// Restore screen attributes

					break;

				case '#':	//

					TFlag = 4;
					return;

				default:

					Break;

			}
			break;

		case 2:

			switch( cChar )	{

				case '@':	// Insert mode

					break;

				case '|':	// Reset mode

					break;

				case '\'':	// Horiz Pos absolute

					break;

				case 'A':	// Cursor up

					break;

				case 'B':	// Cursor Down

					break;

				case 'C':	// Cursor foward

					break;

				case 'D':	// Cursor Backward

					break;

				case 'E':	// Cursor next line

					break;

				case 'F':	// cursor preceding line

					break;

				case 'G':	// Cursor horizontal absolute

					break;

				case 'f':	// Cursor position (x,y)
				case 'H':	

					break;

				case 'J':	// Erase in display

					switch( TFp[0] )	{

						case 0:		// from active position through end

							break;

						case 1:		// from start through active position

							break;

						case 2:		// all of field

							break;

					}

					break;
			
				case 'K':

					switch( TFp[0] )	{

						case 0:

							break;

						case 1:

							break;

						case 2:

							break;

					}
					break;

				case 'L':

					if ( TFp[0] == 0 )
						TFp[0] = 1;

					//	Scroll window down

					break;

				case 'M':

					if ( TFp[0] == 0 )
						TFp[0] = 1;

					// Scroll window up

					break;

				case 'N':	// Erase in field

					break;

				case 'O':	// Erase Area

					break;

				case 'P':	// Delete n chars to left

					if ( TFp[0] == 0 )
						TFp[0] = 1;

					// store the current line of characters of the screen


					break;

				case 'Q':	// Select extent mode

					break;

				case 'R':	// Cursor Report

					break;

				case 'S':	// Scroll Up

					break;

				case 'T':	// Scroll Down

					break;

				case 'U':	// Next Page

					break;

				case 'V':	// Preceding page

					break;

				case 'W':	// Cursor Tab Control

					// 0 Set HT stop at active line
					// 1 Set VT stop at active position
					// 2 Clear HT at active position
					// 3 Clear VT at active position
					// 4 Clear all HT stops in line
					// 5 Clear all HT stops in device
					// 6 Clear all VT stops in device
					
					break;

				case 'X':	// Erase Character

					break;

				case 'Y':	// Cursor Vertical Tab

					break;

				case 'Z':	// Cursor Back Tab

					break;

				case 'a':	// Horiz Position Relative

					break;

				case 'b':	// Repeat char or control

					break;

				case 'c':	// Device Attributes

					break;

				case 'd':	// Vert Position Absolute
				case 'e':	// Vertical Position Relative
				case 'g':	// Tab Clear
				case 'h':	// Select Mode
				case 'l':	// Cursor Horiz. Tab

					break;
				
				case 'i':	// 

					switch( TFp[0] )	{

						case 0:	// Print whole screen

							break;

						case 4:	// exit tranparent printer

							PrtFlag = 0;
							break;

						case 5:	// enter transparent printer

							PrtFlag = 2;
							break;
					}
					break;

				case 'm':	// Adjust video attributes

					i = 0;

					do	{

						switch( TFp[i] )	{
							
							case 0: 
								
								VTa = 0x07;	// Normal Colors

								break;

							case 1:

								VTa |= 8; // High Colors on
								break;

							case 4:
							case 7:

								VTa = 0x70;	// Reverse Video
								break;

							case 3:	// 3 & 6 not supported
							case 6:

								break;

							default:

								if ( TFp[i] >= 40 && TFp[i] <= 47 )
									VTa = ( VTa & 0x87 ) | (( TFp[i] & 7 ) << 4 );
								else
								if ( TFp[i] >= 30 && TFp[i] <= 37 )
									VTa = ( VTa & 0xf8 ) | ( TFp[i] & 7 );

								break;

						}

					} while( ++i <= TFlag1 );

					break;

				case 'n':	// Cirsor report request

					switch( TFp[0] )	{

						case 0:	// Ready
						case 1:	// Busy-retry later
						case 2:	// Busy DTR
						case 3:	// Malfunction
						case 4:	// Malfunction
						case 5:	// Report status via DSR or DSC

							break;

						case 6:	// Report status active cursoe position

							sprintf( buff, ( unsigned char )CURRRPT, VTr, VTc );

							// Send out through the port

							break;

						default:

							break;

					}
					break;

				case 'o':

					// 0 Accept all input
					// 1 Accept no input( protected )
					//	do not xmit ( guarded )
					// 2 Accept graphics
					// 3 Accept numerics
					// 4 Accept Alpha
					// 5 right justify
					// 6 zero-fill
					// 7 HT at start of area
					// 8 accept no input
					// 9 space fill-in area

					break;

				case 'q':

					break;

				case 'r':

					if ( TFp[0] == 0 )	TFp[0] = 1;
					if ( TFp[1] == 0 )	TFp[1] = 24;
					if ( TFp[1] <= TFp[0] )
						break;

					VTwt = TFp[0];
					VTwb = TFp[1];
					if ( VTwb > 24;
						VTwb = 24;

					break;

				case 's':	/// Save cursor position

					SVTc = VTc;
					SVTr = VTr;

					break;

				case 'u':	// restore cursor position

					VTc = SVTc;
					VTr = SVTr;

					// go to x y

					break;

				case 'x':
				case 'y':

					break;

				case '?':

					TFlag = 5;
					return;

				case ';':

					++TFlag1;
					return;

				case '0':
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':

					TFp[TFlag1] *= 10;
					TFp[TFlag1] += ( cChar - '0' );
					return;

			}
			TFlag = 0;
			break;


		case 3:

			TFlag = 0;
			break;

		case 4:

			TFlag = 0;
			break;

		case 5:

			switch( cChar )	{

				case 'h':

					TFlag = 0;
					break;

				case 'l':

					TFlag = 0;
					break;

				case 'i':

					switch( TFp[0] )	{

						case 1:

							break;

						case 4:

							PrtFlag = 0;
							break;

						case 5:

							PrtFlag = 1;
							break;

					}
					break;

				case ';':

					++TFlag1;
					return;

				case '0':
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':

					TFp[TFlag1] *= 10;
					TFP[TFlag] += ( cChar - 10 );
					return;

			}
			TFlag = 0;
			break;

	}
}

//*
//******************************************************************
//*
//* HandleAnsiEmulation
//*
//* Routine to handle standard Ansi 3.2 Emulation.  Interprets screen
//* formatting commands, and color attributes.  Mostly used by 
//* Bulletin Board Systems
//* 
//* Parameters:
//*		HWND			-  Handle of the calling window
//*		int				-  Character to write
//*
//* Returns:
//*		Nothing.
//*
//******************************************************************
//*

void HandleAnsiEmulation( HWND hTTY, int cChar )	{

	static	int		TFlag = 0;
	static	int		TFlag1 = 0;
	static	int		TFp[10];
	static	int		PrtFlag = 0;

	static	int		i;
	static	int		j;

	cChar &= 0x00ff;

	switch( TFlag )	{

		case 0:

			if ( cChar == 0x1b )	{

				for ( TFlag1 = 0; TFlag1 < 10; TFlag1++ )

					TFp[TFlag1] = 0;

				TFlag = 1;
				TFlag1 = 0;
				return;
			}

			switch( cChar )	{

				case 0:

					break;

				case 1:
				case 2:
				case 3:
				case 4:
				case 5:
				case 6:

					break;

				case 7:

					MessageBeep( 0 );
					break;

				case 8:

					break;

				case 9:

					break;

				case 10:

					break;

				case 11:

					break;

				case 12:

					break;

				case 13:

					VTc = 1;
					if ( LFON )	{

						if ( ++VTr > VTwb )	{
							VTr++;

							/// scroll up
						}
					}
					break;

				case 14:	// Use G0 Set
				case 15:	// Use G1 Set

					break;

				case 16:
				case 17:
				case 18:
				case 19:
				case 20:
				case 21:
				case 22:
				case 25:

					break;

				case 24:	// Cancel sequence
				case 26:	// ""		""

					TFlag = 0;
					break;

				case 27:		// Special sequence about to arrive

					for ( TFlag1 = 0; TFlag1 < 10; TFlag1++ )
						TFp[TFlag1] = 0;

					TFlag = 1;
					TFlag1 = 0;
					return;

				case 28:
				case 29:
				case 30:
				case 31:

					// display invalid char

					break;

				default:

					/// display the char

					break;
			}
			break;

		case 1:

			switch( cChar )	{

				case 24:
				case 26:	// Cancel Sequence

					TFlag = 0;
					break;

				case 27:	// an extra escape char

					return;

				case '[':

					TFlag = 2;
					TFlag1 = 0;
					return;

				case '|':

					break;

				case '(':
				case ')':

					TFlag = 3;
					return;

				case 0x12:

					TFlag = 0;
					break;

				case 0x0d:

					VTc = 1;
					break;

				case 'D':

					if ( ++VTr > VTwb )	{
						VTr = VTwb;
						// Scroll up a line
					}
					break;

				case 'E':

					VTc = 1;
					if ( ++Vtr > VTwb )	{
						VTr = VTwb;
						// Scroll up a line
					}
					break;

				case 'F':	// Start of selected area

					break;

				case 'G':	// End of Selected area

					break;
			
				case 'H':	// Horiz. Tab Set

					break;

				case 'L':	// Partial Line Down

					break;

				case 'K':	// Partial line up

					break;

				case 'M':	// Reverse Index

					if ( --VTr < VTwt )	{
						VTr = VTwt;
						//Scroll down a line
					}
					break;

				case 'U':

					break;

				case 'V':

					break;

				case 'W':

					break;

				case 'Z':

					break;

				case 'c':
				case '=':
				case '>':

					break;

				case '7':

					SVTr = VTr;
					SVTc = VTc;
					SVTa = VTa;
					break;

				case '8':

					VTr = SVTr;
					VTc = SVTc;
					VTa = SVTa;
					break;

				case '#':

					TFlag = 4;
					return;

				default:

					break;

			}
			TFlag = 0;
			break;

			// Escape sequence detected...

		case 2:

			switch( cChar )	{

				case '@':	// Insert Character

					break;

				case '|':	// Reset mode

					break;

				case '\'':	// Horiz position absolute

					break;

				case 'A':	// Cursor up

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					while( TFp[0]-- > 0 )
						if ( VTr > VTwt )
							VTr--;
					break;

				case 'B':	// Cursor down

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					while( TFp[0]-- > 0 )
						if ( VTr < VTwb )
							VTr++;
					break;

				case 'C':	// Cursor Foward

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					while( TFp[0]-- > 0 )
						if ( VTc < 80 )
							VTc++;
					break;

				case 'D':	// Cursor Backward

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					while( TFp[0]-- > 0 )
						if ( VTc > 1)
							VTc--;
					break;

				case 'E':	// Cursor Next Line

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					while( TFp[0]-- > 0 )
						if ( VTr < VTwb )
							VTr++;
					break;

				case 'F':	// Cursor Preceding Line

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					while( TFp[0]-- > 0 )
						if ( VTr < VTwt )
							VTr++;
					break;

				case 'G':	Cursor Horiz. Absolute

					if ( TFp[0] == 0 )
						TFp[0] = 1;

					if ( TFp[0] > 80 )
						TFp[0] = 80;
					VTc = TFp[0];

					break;

				case 'f':
				case 'H':	// Cursor position x,y

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					if ( TFp[1] == 0 )
						TFp[1] = 1;
					VTr = TFp[0];
					if ( VTr > 24 )
						VTr = 24;
					VTc = TFp[1];
					if ( VTc > 80 )
						VTc = 80;
					break;

				case 'J':	// Erase in Display

					switch( TFp[0] )	{

						case 0:	// From active position through end

						case 1:	// From start though active position

						case 2:	// All of field
							break;
					}
					break;

				case 'K':	// Erase in Line

					switch( TFp[0] )	{

						case 0:	// From active position through end
						case 1:	// from start though active position
						case 2:	// All of line
							break;

					}
					break;

				case 'L':	// Insert line

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					// ScrollWindowdown

					break;

				case 'M':	// Delete line

					if ( TFp[0] == 0 )
						TFp[0] = 1;
					// Scrollwindow up
					
					break;

				case 'N':	// Erase in field

					break;

				case 'O':	// Erase Area

					break;

				case 'P':	// Delete n chars to left

					if ( TFp[0] == 0 )
						TFp[0] = 1;

					// first we need to store contents of the current line

					// Shift characters in buffer left n times...

					break;

				case 'Q':

					break;

				case 'R':	// Cursor report

					break;

				case 'S':	// Scroll up

					if ( TFp[0] == 0 )
						TFp[0] = 1;

					// Scroll page up

					break;

				case 'T':	// Scroll Down

					if ( TFp[0] == 0 )
						TFp[0] = 1;

					// Scroll page down

					break;

				case 'U':	// Next page

					break;

				case 'V':	// Preceding page

					break;

				case 'W':	// Cursor tab control

					// 0 Set HT stop active line
					// 1 Set VT stop at active position
					// 2 Clear HT at active position
					// 3 Clear VT at active line
					// 4 Clear all HT stops in line
					// 5 Clear all HT stops in device
					// 6 Clear all VT stops in device

					break;

				case 'X':	// Erase character

					break;

				case 'Y':	// Cursor Vertical Tab

					break;

				case 'Z':	// Cursor Back Tabulation

					break;

				case 'a':	// Horiz. position relative

					break;

				case 'b':	// Repeat char or control

					break;

				case 'c':	// Device attributes

					// Send "\033[?6c\0";

					break;

				case 'd':
				case 'e':
				case 'g':
				case 'h':
				case 'l':

					break;

				case 'i':	// Print Control

					switch ( TFp[0] )	{

						case 0:	// Print whole screen

							break;

						case 4:	// Exit transparent printer

							break;

						case 5:	// enter transparent printer

							break;

					}
					break;

				case 'm':

					i = 0;
					do	{
						switch( TFp[0] )	{

							case 0:				// Reset to normal

								VTa = 0x07;	
								break;

							case 1:				// High colors on

								VTa |= 0x08;	
								break;

							case 4:				// Set reverse video

								VTa = 0x70;		
								break;

							case 7:				// set reverse video

								VTa = 0x70;
								break;

							case 2:				// Faint
							case 3:				// Italic
							case 6:				// Rapid blink
							case 8:				// Reserved
							case 9:				// Reserved
							case 10:			// Primary font as designated
												// by < font selection >
							case 11:			// First Alt. font
							case 12:
							case 13:
							case 14:
							case 15:
							case 16:
							case 17:
							case 18:
							case 19:			// Ninth Alt. font
							case 20:			// Fraktur

								break;

							default:

								if ( TFp[i] >= 40 && TFp[i] <= 47 )	{

									atr = TFp[i] - 40;
									switch( atr )	{

										case 1:

											atr = 0x04;
											break;

										case 3:

											atr = 0x06;
											break;

										case 4:

											atr = 0x01;
											break;

										case 6:

											atr = 0x03;
											break;

										default:

											break;

									}

									VTa = VTa & 0x8f | ( atr << 4 );
								}
								else
									if ( TFp[i] >= 30 && TFp[i] <= 37 )	{

										atr = TFp[i] - 30;








}


void DisplayScreenChar( HWND hTTY, int cChar )	{

	RECT	rect;

	SCREENCHAR( TTYInfo, COLUMN( TTYInfo ), ROW( TTYInfo )) = (char)cChar;

	rect.left = ( COLUMN( TTYInfo ) * XCHAR( TTYInfo )) -
		XOFFSET( TTYInfo );

	rect.right = rect.left + XCHAR( TTYInfo );
	rect.top = ( ROW( TTYInfo ) * YCHAR( TTYInfo )) -
		YOFFSET( TTYInfo );

	rect.bottom = rect.top + YCHAR( TTYInfo );
	InvalidateRect( hTTY, &rect, FALSE );

	//
	// Line Wrap
	//

	if ( COLUMN( TTYInfo ) < MAXCOLS - 1 )
		COLUMN( TTYInfo )++;
	else if ( AUTOWRAP( TTYInfo ))
		OutputABufferToWindow( hTTY, "\r\n", 2 );

}

//*
//******************************************************************
//*
//******************************************************************
//*


void OutputABufferToFile( HANDLE hFile, char * lpBuf, DWORD dwBufLen )	{

//	DWORD		dwWritten;

	//
	// place buffer into file, report any errors
	//

//	if ( !WriteFile( hFile, lpBuf, dwBufLen, &dwWritten, NULL ))

//		ErrorReporter( "WriteFile in file capture" );

//	if ( dwBufLen != dwWritten )

//		ErrorReporter( "WriteFile" );

//	PostMessage( GetDlgItem( ghWndStatusDlg, IDC_TRANSFERPROGRESS, PBM_STEPIT, 0, 0 );

	return;
}


//*
//******************************************************************
//*
//******************************************************************
//*


void OutputABuffer( HWND hTTY, char * lpBuf, DWORD dwBufLen )	{

//	if ( dwBufLen == 0 )	{

//		OutputDebugString( "NULL Buffer in OutputABuffer\n\r" );
//		return;
//	}

//	switch( gdwReceiveState )	{

//		case RECEIVE_TTY:

//			OutputABufferToWindow( hTTY, lpBuf, dwBufLen );
//			break;

//		case RECEIVE_CAPTURE:

//			OutputABufferToFile( ghFileCapture, lpBuf, dwBufLen );
//			break;

//		default:

//			OutputDebugString( Unknown receive state!\n\r" );
//	}

	return;

}




