//*
//******************************************************************
//* Module: UserPref.c
//*
//*		This module handles all issues with loading/saving
//*	the user settings & phonebooks...
//*
//*	Author:
//*				Byron D Copeland
//*	Date:		September 6, 1999
//*
//******************************************************************
//*
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <io.h>
#include "UserPref.h"
#include "Database.h"
#include "Globals.h"

//*
//* Local variable declarations
//*

HANDLE				hSettingsFile = INVALID_HANDLE_VALUE;
HANDLE				hPBFile = INVALID_HANDLE_VALUE;
HANDLE				hTempFile = INVALID_HANDLE_VALUE;

//*
//* Local prototpe declarations
//*

LPARAM LoadUserSettings( HWND, WORD, HWND, WORD );
LPARAM SaveUserSettings( HWND, WORD, HWND, WORD );
LPARAM SelectPhoneBook( HWND, WORD, HWND, WORD );
LPARAM SavePhoneBook( HWND, WORD, HWND, WORD );
LPARAM ImportPhoneFile( HWND, WORD, HWND, WORD );

INT ScanModemConfigurationKeys( HWND );

//*
//* External functions not declared in this module...
//*

void InitDatabase( void );
void InitDatabaseItem( INT, LPTSTR, LPTSTR, LPTSTR, LPTSTR, INT );
void SetPhoneBookLoaded( BOOL );
INT GetMaxListViewItems( void );

//*
//******************************************************************
//* DoMainLoadUserSettings
//*
//* Allows retrieval of user settings...
//******************************************************************
//*

LPARAM LoadUserSettings( HWND hWnd, WORD idItem, HWND hwndCtl,
							  WORD wNotifyCode )	{

	OPENFILENAME		ofn;
	HANDLE				hLoadHandle;
	TCHAR				szSelectedFile[MAX_PATH] = {0};
	TCHAR				szTmp[128] = TEXT( "" );
	DWORD				dwBytesRead;
	DWORD				dwFileSize;
	LPSTR				lpInitialDir = TEXT( "C:\\Kalimba\\Config" );

	const LPTSTR		lpDefExt = TEXT( "kfg" );
	const LPTSTR		lpDefFile = TEXT( "*.kfg" );

	const LPTSTR		lpOpenFilterNames = 
		TEXT( "Kalimba Config Files (*.kfg)\0*.kfg\0All Files (*.*)\0*.*\0\0" );

	const LPTSTR		lpDialogTitle = TEXT( "Load Settings" );

	memset( &ofn, 0, sizeof( ofn ));

	ofn.lStructSize = sizeof( ofn );
	ofn.hwndOwner = hWnd;
	ofn.lpstrTitle = lpDialogTitle;
	ofn.lpstrInitialDir = lpInitialDir;
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
	ofn.lpstrDefExt = lpDefExt;
	ofn.lpstrFilter = lpOpenFilterNames;
	ofn.lpstrFile = szSelectedFile;
	ofn.nFilterIndex = 1;
	ofn.nMaxFile = dim( szSelectedFile );

	if ( GetOpenFileName( &ofn ))	{

		// Open and read in the user default settings...

		if ( lstrlen( szSelectedFile ) != 0 )	{

			hLoadHandle = CreateFile( szSelectedFile, GENERIC_READ,
								FILE_SHARE_WRITE | FILE_SHARE_READ, 
								NULL, OPEN_EXISTING, 
								FILE_ATTRIBUTE_NORMAL, NULL );

			if ( hLoadHandle == INVALID_HANDLE_VALUE )	{
//				HandleErrorCode( hWnd, szTmp, "Load Settings" );

			}
			else	{
				dwFileSize = GetFileSize( hSettingsFile, NULL );
				if ( !ReadFile( hLoadHandle, &userSavedSettings, 
							sizeof( userSavedSettings ), &dwBytesRead, NULL ))	{

//					HandleErrorCode( hWnd, szTmp, "Load Settings" );

				}
				CloseHandle( hSettingsFile );
				hLoadHandle = INVALID_HANDLE_VALUE;

			}
		}
	}
	ScanModemConfigurationKeys( hWnd );
	return 0;
}

//
//******************************************************************
//* DoMainSaveUserSettings
//*
//* Allows saving of all user settings...
//******************************************************************
//
LPARAM SaveUserSettings( HWND hWnd, WORD idItem, HWND hwndCtl,
							  WORD wNotifyCode )	{

	OPENFILENAME		ofn;
	HANDLE				hSaveHandle;
	DWORD				dwBytesWritten;
	TCHAR				szSelectedFile[MAX_PATH];
	TCHAR				szFileOnly[MAX_PATH];
	TCHAR				szTmp[128] = TEXT( "" );
	LPSTR				lpInitialDir = TEXT( "C:\\Kalimba\\Config" );

//	const LPTSTR		lpInitFile = TEXT( "MySettings.kfg" );

	const LPTSTR		lpDialogTitle = TEXT( "Save Settings" );
	const LPTSTR		lpDefExt = TEXT( "kfg" );
	const LPTSTR		lpDefFile = TEXT( "*.kfg" );
	const LPTSTR		lpOpenFilterNames = 
		TEXT( "Kalimba Files (*.kfg)\0*.kfg\0All Files (*.*)\0*.*\0\0" );


	// Setup for initial display of SaveAs dislog...use filename
	// last loaded for initial display...if non, then use
	// Kalimba.kfg for a file name...
	// The file name entered by the user can be any they choose
	// but, the extension should always remain *.kfg...

	szSelectedFile[0] = '\0';
	memset( &ofn, 0, sizeof( ofn ));

	ofn.lStructSize = sizeof( ofn );
	ofn.hwndOwner = hWnd;
	ofn.lpstrTitle = lpDialogTitle;
	ofn.lpstrFile = szSelectedFile;
	ofn.nMaxFile = dim( szSelectedFile );
	ofn.lpstrFileTitle = szFileOnly;
	ofn.nMaxFileTitle = dim( szFileOnly );
	ofn.lpstrInitialDir = lpInitialDir;
	ofn.nFilterIndex = 1;
	ofn.Flags = OFN_CREATEPROMPT;
	ofn.lpstrDefExt = lpDefExt;
	ofn.lpstrFilter = lpOpenFilterNames;

	hSaveHandle = INVALID_HANDLE_VALUE;

	if ( GetSaveFileName( &ofn ))	{

		// 
		// Go create file and save user default settings...if we have a file
		// name...
		//

		hSaveHandle = CreateFile( szSelectedFile, GENERIC_WRITE, 
				FILE_SHARE_WRITE | FILE_SHARE_READ, NULL, 
				CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );

		if ( hSaveHandle != INVALID_HANDLE_VALUE )	{

			if ( !WriteFile( hSaveHandle, &userSavedSettings, 
					sizeof( userSavedSettings ), &dwBytesWritten, NULL ))	{

//				HandleErrorCode( hWnd, szTmp, "Save Settings" );

			}
			else	{

				// Reset all the properties change flags...

				fCommBasicPropertiesChanged = FALSE;
				fCommInitsPropertiesChanged = FALSE;
				fCommMessagesPropertiesChanged = FALSE;
				fCommAdvPropertiesChanged = FALSE;

			}
			CloseHandle( hSaveHandle );
			hSaveHandle = INVALID_HANDLE_VALUE;
		}
		else	{

//			HandleErrorCode( hWnd, szTmp, "Save Setings" );
		}
	}

	return 0;
}

//
//******************************************************************
//* DoMainSelectPhoneBook
//*
//* Process the select phone book menu option
//******************************************************************
//
LPARAM SelectPhoneBook( HWND hWnd, WORD idItem, HWND hwndCtl,
						 WORD wNotifyCode )	{

	OPENFILENAME	ofn;

	TCHAR			szSelectedFile[MAX_PATH] = {0};
	TCHAR			szTmp[100];
	UINT			MAXTOREAD = 0;
	DWORD			dwBytesRead;
	DWORD			dwFileSize;
	UINT			i;
	LPSTR			lpInitialDir = TEXT( "C:\\Kalimba\\Config" );
	
	const LPTSTR	lpDialogTitle = TEXT( "Open Phone Book" );
	const LPTSTR	lpDefExt = TEXT( "kpb" );
	const LPTSTR	lpOpenFilterNames = 
			TEXT( "Kalimba PhoneBook (*.kpb)\0*.kpb\0\0" );

	memset( &ofn, 0, sizeof( ofn ));

	ofn.lStructSize = sizeof( ofn );
	ofn.hwndOwner = hWnd;
	ofn.lpstrTitle = lpDialogTitle;
	ofn.lpstrInitialDir = lpInitialDir;
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
	ofn.lpstrDefExt = lpDefExt;
	ofn.lpstrFilter = lpOpenFilterNames;
	ofn.lpstrFile = szSelectedFile;
	ofn.nFilterIndex = 1;
	ofn.nMaxFile = dim( szSelectedFile );


	if ( GetOpenFileName( &ofn ))	{

		if ( hPBFile != INVALID_HANDLE_VALUE )	{
			CloseHandle( hPBFile );
			hPBFile = INVALID_HANDLE_VALUE;
		}

		if ( lstrlen( szSelectedFile ) != 0 )	{

			hPBFile = CreateFile( szSelectedFile, GENERIC_READ,
								FILE_SHARE_READ, NULL, OPEN_EXISTING, 
								FILE_ATTRIBUTE_NORMAL, NULL );

			if ( hPBFile == INVALID_HANDLE_VALUE )	{
				MessageBox( hWnd, "File Open Error!", "File Open", 0 );
			}
			else	{
				dwFileSize = GetFileSize( hPBFile, NULL );
				lstrcpy( userSavedSettings.szLastPhoneBookUsed, szSelectedFile );
				MAXTOREAD = dwFileSize / sizeof( LVDATAITEM );
				wsprintf( szTmp, "Items to Read: %u", MAXTOREAD );
				for ( i = 0; i < MAXTOREAD; i++ )	{
					if ( !ReadFile( hPBFile, &lvDialDirectory[i], sizeof( LVDATAITEM ),
						&dwBytesRead, NULL ))	{

						// Handle Error
						SetPhoneBookLoaded( FALSE );
					}
					else	{
						SetPhoneBookLoaded( TRUE );
					}
				}
				CloseHandle( hPBFile );
				hPBFile = INVALID_HANDLE_VALUE;
			}
		}
	}
	return 0;
}

//
//******************************************************************
// DoMainSavePhoneBook
//
// Save the currently opened phone book
//******************************************************************
//
LPARAM SavePhoneBook( HWND hWnd, WORD idItem, HWND hwndCtl,
						   WORD wNotifyCode )	{

	OPENFILENAME		ofn;
	HANDLE				hSaveHandle;
	DWORD				dwBytesWritten;
//	TCHAR				szFileOnly[MAX_PATH];
	TCHAR				szTmp[128] = TEXT( "" );
	TCHAR				szSelectedFile[MAX_PATH] = {0};
	LPSTR				lpInitialDir = TEXT( "C:\\Kalimba\\Config" );
	INT					i;
//	const LPTSTR		lpInitFile = TEXT( "MySettings.kfg" );
	const LPTSTR		lpDialogTitle = TEXT( "Save Phone Book" );
	const LPTSTR		lpDefExt = TEXT( "kpb" );
	const LPTSTR		lpDefFile = TEXT( "*.kpb" );
	const LPTSTR		lpOpenFilterNames = 
		TEXT( "Kalimba Files (*.kpb)\0*.kpb\0All Files (*.*)\0*.*\0\0" );

	memset( &ofn, 0, sizeof( ofn ));

	ofn.lStructSize = sizeof( ofn );
	ofn.hwndOwner = hWnd;
	ofn.lpstrTitle = lpDialogTitle;
	ofn.lpstrInitialDir = lpInitialDir;
	ofn.Flags = OFN_PATHMUSTEXIST;
	ofn.lpstrDefExt = lpDefExt;
	ofn.lpstrFilter = lpOpenFilterNames;
	ofn.lpstrFile = szSelectedFile;
	ofn.nFilterIndex = 1;
	ofn.nMaxFile = dim( szSelectedFile );


	hSaveHandle = INVALID_HANDLE_VALUE;

	if ( GetSaveFileName( &ofn ))	{


		hSaveHandle = CreateFile( szSelectedFile, GENERIC_WRITE, 
				FILE_SHARE_WRITE | FILE_SHARE_READ, NULL, 
				CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL );

		if ( hSaveHandle != INVALID_HANDLE_VALUE )	{

			lstrcpy( userSavedSettings.szLastPhoneBookUsed, szSelectedFile );
			for ( i = 0; i < GetMaxListViewItems(); i++ )	{
				if ( lstrlen( lvDialDirectory[i].Number ) > 0 )	{
					if ( !WriteFile( hSaveHandle, &lvDialDirectory[i], 
						sizeof( LVDATAITEM ), &dwBytesWritten, NULL ))	{


//						HandleErrorCode( hWnd, szTmp, "Save Phone Book" );
					}
				}

			}
			CloseHandle( hSaveHandle );
			hSaveHandle = INVALID_HANDLE_VALUE;
		}
		else	{

//			HandleErrorCode( hWnd, szTmp, "Save Phone Book" );
		}

		// Reset all the properties change flags...
	}

	return 0;
}
//
//******************************************************************
//* DoMainImportPhoneFile
//*
//* Allows a user to import an ascii text phone book...
//******************************************************************
//
LPARAM ImportPhoneFile( HWND hWnd, WORD idItem, HWND hwndCtl,
							 WORD wNotifyCode )	{

	OPENFILENAME		ofn;

	TCHAR				szSelectedFile[MAX_PATH] = {0};
	TCHAR				szBuf[128] = TEXT("");
	TCHAR				szText[256] = TEXT( "" );
	TCHAR				szNumber[100];
	TCHAR				szDesc[100];
	DWORD				dwFileSize;
	DWORD				nBytesRead = 0;
	DWORD				nTotalBytesRead = 0;
	BOOL				bResult;
//	INT					i;
	INT					nPBIndex = 0;
	char				*p;
	char				*q;

	const LPTSTR		lpInitialDir = TEXT( "C:\\Kalimba\\Config" );
	const LPTSTR		lpDefExt = TEXT( "apb" );
	const LPTSTR		lpDialogTitle = TEXT( "Import Ascii Phone Book" );
	const LPTSTR		lpOpenFilterNames = 
		TEXT( "Ascii Phone Book (*.apb)\0*.apb\0All Files (*.*)\0*.*\0\0" );

	memset( &ofn, 0, sizeof( ofn ));

	ofn.lStructSize = sizeof( ofn );
	ofn.hwndOwner = hWnd;
	ofn.lpstrTitle = lpDialogTitle;
	ofn.lpstrInitialDir = lpInitialDir;
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
	ofn.lpstrDefExt = lpDefExt;

	ofn.lpstrFilter = lpOpenFilterNames;
	ofn.lpstrFile = szSelectedFile;
	ofn.nFilterIndex = 1;
	ofn.nMaxFile = dim( szSelectedFile );

	if ( GetOpenFileName( &ofn ))	{

		if ( hTempFile != INVALID_HANDLE_VALUE )	{
			CloseHandle( hPBFile );
			hTempFile = INVALID_HANDLE_VALUE;
		}

		if ( lstrlen( szSelectedFile ) != 0 )	{

			fPhoneBookLoaded = FALSE;
			InitDatabase();
			gnTotalPhoneDirRecords = 0;

			hTempFile = CreateFile( szSelectedFile, GENERIC_READ,
								FILE_SHARE_WRITE | FILE_SHARE_READ, 
								NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );

			if ( hTempFile == INVALID_HANDLE_VALUE )	{

//				HandleErrorCode( hWnd, szBuf, "Import" );

			}
			else	{

				dwFileSize = GetFileSize( hTempFile, NULL );
				lstrcpy( userSavedSettings.szLastPhoneBookUsed, szSelectedFile );

				//
				// Display Progress Bar
				//

				while( nTotalBytesRead < dwFileSize )	{

					bResult = ReadFile( hTempFile, &szBuf, 1, &nBytesRead, NULL );
					if ( bResult && nBytesRead == 0 )	{

						// Scan data and place in our struture

//						SendDlgItemMessage( hWnd, ID_RCVTEXT, EM_REPLACESEL, 0,
//							(LPARAM)szText );

						lstrcat( szText, szBuf );
						break;
					}
					nTotalBytesRead += nBytesRead;
					if ( szBuf[0] == '\n' )	{
						lstrcat( szText, "\0" );

						// Scan data and place in our structure...

//						scanf( szBuf, "%s %s", &szPhoneNum, &szDesc );

						
						if (( p = strstr( szText, "\n" )) != NULL )
							*p = '\0';

						if (( p = strstr( szText, "\r" )) != NULL )
							*p = '\0';

						p = szText;
						q = szText;

						//
						// Seperator character is a comma(,) at the moment
						// the first encountered in the string
						//
						while( *q && *q != ',' )	{
							q++;
						}
						*q = '\0';
						q++;
						lstrcpy( szDesc, q );

						lstrcpy( szNumber, p );

						
//						wsprintf( szText, "%s --- %s", szNumber, szDesc );
//						MessageBox( hWnd, szText, "Imported", 0 );
						InitDatabaseItem( nPBIndex, szNumber, szDesc, "", "", nPBIndex );
						nPBIndex++;

//						gnTotalPhoneDirRecords += 1;

//						SendDlgItemMessage( hWnd, ID_RCVTEXT, EM_REPLACESEL, 0,
//							(LPARAM)szDesc );

						memset( szText, 0, sizeof( szText ));
					}
					else	{
						lstrcat( szText, szBuf );
					}
				}
				CloseHandle( hTempFile );
				hTempFile = INVALID_HANDLE_VALUE;
			}
//			wsprintf( szText, "Total Loaded %ld", gnTotalPhoneDirRecords );
//			MessageBox( hWnd, szText, "Records", 0 );
		}
	}

	return 0;
}


INT ScanModemConfigurationKeys( HWND hWnd )	{


	HKEY	hKey = HKEY_LOCAL_MACHINE;
	LPCTSTR lpSubKey = NULL;		// address of name of subkey to open
	DWORD ulOptions = 0;			// reserved
	REGSAM samDesired = KEY_READ;	// security access mask
	HKEY phkResult;				// address of handle to open key

	LPTSTR lpValueName = TEXT( "Software" );  // address of name of value to query
	DWORD lpReserved = 0;  // reserved
	DWORD lpType;      // address of buffer for value type
	BYTE *lpData;       // address of data buffer
	DWORD lpcbData;
	TCHAR	szTmp[30] = TEXT( "" );

	//*
	//* Open the HKEY_LOCAL_MACHINE to allow us access to
	//* the modem configuration info.
	//*

	if ( RegOpenKeyEx( hKey, lpSubKey, 
		ulOptions, samDesired, &phkResult ) == ERROR_SUCCESS )	{
//		MessageBox( hWnd, "GOT REGISTRY OPEN OK!", "REGISTRY", 0 );

		//*
		//* Now Search for the MODEMS....
		//*
		RegQueryValueEx( phkResult, lpValueName, &lpReserved, &lpType,
			&lpData, &lpcbData );

		wsprintf( szTmp, "%u", lpcbData );
//		MessageBox( hWnd, szTmp, "REGISTRY", 0 );
		RegCloseKey( phkResult );
	}
	return 0;

}